
------------------------------------
-----------[[  MODEL  ]]------------
----state, logic, entities----------
------(sole caller of the view)-----
------------------------------------

------------
---- Globals
--------------------


Archimedes.debugging = false
function Archimedes.Debug(input)
  if Archimedes.debugging then
    print(input)
  end
end
------------------------------------
----------[[  MODEL:ARCS  ]]--------
-----arc creation and handling------
----------(add/remove/copy)---------
------------------------------------




------------------------------------
--------[[  MODEL:EVENTS  ]]--------
---------Handles all input----------
-----(change in state/condition)----
------------------------------------
Archimedes.changeEvent = {}

------------------------------------
-----[[  MODEL:EVENTS:CHANGE  ]]----
------Calls actions from events-----
-----(checks condition settings)----
------------------------------------

-- To be continued...






function Archimedes.Visibility()
  Archimedes.side.left:SetVisible(not Archimedes.Options.globals.disabled)
  Archimedes.side.right:SetVisible(not Archimedes.Options.globals.disabled)
end


function Archimedes.Init()
  ----Make sure that all settings exist in the options
  for arc,opt in pairs(Archimedes.Options.arcs) do
    --if not exists, then new
    if not opt.specifier then Archimedes.Options.arcs[arc].specifier = Archimedes.defaultOptions.arc.specifier end
    if not opt.offsetx then Archimedes.Options.arcs[arc].offsetx = Archimedes.defaultOptions.arc.offsetx end
    if not opt.offsety then Archimedes.Options.arcs[arc].offsety = Archimedes.defaultOptions.arc.offsety end
    if not opt.scale then Archimedes.Options.arcs[arc].scale = Archimedes.defaultOptions.arc.scale end
    if not opt.alpha then Archimedes.Options.arcs[arc].alpha = Archimedes.defaultOptions.arc.alpha end
    if not opt.skinPack then opt.skinPack = opt.foregroundPack or Archimedes.defaultOptions.arc.skinPack end
    -- if old exists, then transfer, otherwise new
    -- old,
    if opt.foregroundTexture then opt.texture = opt.foregroundTexture; opt.foregroundTexture = nil end
    if opt.foregroundID then opt.texture = opt.foregroundID; opt.foregroundID = nil end
    if not opt.texture then opt.texture = Archimedes.defaultOptions.arc.texture end

  end
  if not Archimedes.Options.globals then Archimedes.Options.globals = {} end
  if not Archimedes.Options.globals.disabled then Archimedes.Options.globals.disabled = Archimedes.defaultOptions.globals.disabled end
   if not Archimedes.Options.globals.combatAlpha then Archimedes.Options.globals.combatAlpha = Archimedes.defaultOptions.globals.combatAlpha end
  if not Archimedes.Options.globals.spacing then Archimedes.Options.globals.spacing = Archimedes.defaultOptions.globals.spacing end
  if not Archimedes.Options.globals.alpha then Archimedes.Options.globals.alpha = Archimedes.defaultOptions.globals.alpha end
  Archimedes.View()
  Archimedes.ArcLoader()
  Archimedes.Visibility()
  Archimedes.initialised = true
end


------------
---- Utilities
--------------------


function Archimedes.deepcopy(object)
    local lookup_table = {}
    local function _copy(object)
        if type(object) ~= "table" then
            return object
        elseif lookup_table[object] then
            return lookup_table[object]
        end
        local new_table = {}
        lookup_table[object] = new_table
        for index, value in pairs(object) do
            new_table[_copy(index)] = _copy(value)
        end
        return setmetatable(new_table, getmetatable(object))
    end
    return _copy(object)
end



function Archimedes.newName(nameStart)
  local availName = nameStart
  local avail = true -- True if our string does NOT correspond to any of the names

  -- First! Check all arcs, if the name is nameStart (set true until)
  for arc,opt in pairs(Archimedes.Options.arcs) do
    if avail == true then
      if opt.name == availName then
        avail = false -- We have found one named nameStart so we will exit stop looking
      else
       avail = true -- If we never find it and cancel, we know it's available
      end
    end
  end
  --Second!
  --As long as we have not found an available name, iterate through all arcs and test it against our wanted name
  local num = 1
  for i = 1, 20  do
    if avail == false then
    availName = nameStart .. " (" .. string.format("%02d", tostring(num)) .. ")"
    for arc,opt in pairs(Archimedes.Options.arcs) do   --check every
      if avail == true then
        print("This will not happen")
        if opt.name == availName then
          avail = false
        else
          avail = true
        end
      end
    end --end iteration
    num = num + 1 -- Add one and continue loop
    --if num > 15 then break end
    end
  end --end loop
  return availName
end
----------------
---- Slash Commands
-----------------------
function Archimedes.SlashHandler(args)
local function CommandLine(str)
  local result = {}
      local count = 0
      for token in string.gmatch(str, "[^%s]+") do
        count = count + 1
        result[count] = token
      end
      return result[1], result[2], result[3], result[4], result[5], result[6], result[7], result[8]

end
local cmd, a, b, c, d, e, f, g = CommandLine(args)
if cmd == "help" then
print("Help coming soon!")
elseif cmd == "settings" then
  Archimedes.Settings:toggle()
elseif cmd == "debug" then
  Archimedes.debugging = not Archimedes.debugging
  print("debugging is " .. ((Archimedes.debugging and "ON") or (not Archimedes.debugging and "OFF")))
else
  Archimedes.Settings:toggle()
end
end

----------------
---- Arcs!
---------------------------
-- These are the frames

Archimedes.activeArcs = {}
Archimedes.zombieArcs = {}

-------------------
---- Handle arcs and settings
------------------------------------
function Archimedes.NewArc ()
  local opt = Archimedes.deepcopy(Archimedes.defaultOptions.arc)
  opt.name = "New Arc"
  return Archimedes.AddArc(opt)
end

function Archimedes.CopyArc (arc)
  local opt = Archimedes.deepcopy(Archimedes.Options.arcs[arc])
  opt.name = opt.name .. " copy"
  return Archimedes.AddArc(opt)
end


function Archimedes.NewId()
  local idnum = 0
  local avail = false
  while avail == false do
  if Archimedes.Options.arcs["arc" .. idnum] then
    avail =  false
    idnum = idnum + 1
  else
    return "arc" .. idnum
  end
  end
end

function Archimedes.AddArc (opt)
  -- Creates arcs
  local newid = Archimedes.NewId()
  Archimedes.Options.arcs[newid] = opt
  Archimedes.ArcLoader()
  return newid
end

function Archimedes.RemoveArc (arc)
 Archimedes.Options.arcs[arc] = nil
 Archimedes.activeArcs[arc]:SetVisible(false)
 table.insert(Archimedes.zombieArcs, Archimedes.activeArcs[arc])
 Archimedes.activeArcs[arc] = nil
 Archimedes.ArcLoader()
end

----------------
---- Arc conditions and visibility
-------------------------------------------------------------



function Archimedes.CondCheck(arc) -- checks all active conditions and shows/hides accordingly
  for condition, value in pairs(Archimedes.activeArcs[arc].conditions) do
    Archimedes.activeArcs[arc]:ChangeCondition(condition, value)
  end
end



function Archimedes.ShowHide(arc, unit, type, specifier) -- arc = the ID of the arc that is to be shown/hidden, unit = a unit ID or nil (for hiding), type = castbar/stat, specifier = specifier for stat checking
-- If we have a unit, then do more checks
  if unit then
  local details = {}
  if Inspect.Unit.Lookup(unit) then details = Inspect.Unit.Detail(unit) end
  if not details then return end
  if type == "stat" then
    if details[specifier] then
      Archimedes.activeArcs[arc]:SetVisible(true)
      Archimedes.activeArcs[arc]:ForceUpdate()
    else
    -- Hide the arc if there's no data for it
    Archimedes.activeArcs[arc]:SetVisible(false)
    end
  elseif type =="castbar" then
    local active = Inspect.Unit.Castbar(unit)
    Archimedes.Castbar.ShowHide(unit, arc, active, Inspect.Unit.Lookup(unit))
  end
  else -- Else, hide the arc
  Archimedes.activeArcs[arc]:SetVisible(false)
  end
end

----------------
---- Create arcs (this is the visual arc creation)
-------------------------------------------------------------



function Archimedes.UnitChangeEventHandler(unit, unitSpecifier)
  for arc,opt in pairs(Archimedes.Options.arcs) do
    if unitSpecifier == opt.unit then
      Archimedes.ShowHide(arc, unit, opt.type, opt.specifier)
    end
  end
end

function Archimedes.ArcLoader()
  for arc,opt in pairs(Archimedes.Options.arcs) do
    -- Create Frames for each arc in Archimedes.Options.arcs
    if not Archimedes.activeArcs[arc] then
      Archimedes.activeArcs[arc] = Archimedes.CreateArc(arc, opt)
    end
    -- register the event for each unit
    Archimedes.changeEvent[opt.unit] = Library.LibUnitChange.Register(opt.unit)
    local overwritten = false
    for i,v in pairs(Archimedes.changeEvent[opt.unit]) do
      if v[3] == opt.unit .. " EventHandler" then
        v = {function (upunit)
        -- unit either returns as a value or false
        Archimedes.UnitChangeEventHandler(upunit, opt.unit)
        end , "Archimedes", opt.unit .. " EventHandler" }
        overwritten = true
      end
    end
    if not overwritten then
      table.insert(Archimedes.changeEvent[opt.unit], {function (upunit)
        Archimedes.UnitChangeEventHandler(upunit, opt.unit)
      end , "Archimedes", opt.unit .. " EventHandler"})
    end
    --Inspect.Unit.Lookup(opt.unit) either returns a value or nil
    Archimedes.activeArcs[arc]:ReloadSettings()
    Archimedes.ShowHide(arc, Inspect.Unit.Lookup(opt.unit), opt.type, opt.specifier)
  end
end


-----------------------------------
---- Update Handler (handles updated data)
------------------------------------------------------

function Archimedes.StatHandler(data, specifier, max)
  for unit,value in pairs(data) do --For each updated unit
    for arc,opt in pairs(Archimedes.Options.arcs) do -- find all arcs using the selected specifier for the selected unit
      if opt.type == "stat" then
      if Inspect.Unit.Lookup(opt.unit) == unit then -- make sure to compare the raw units and not the specifier, duh!
        if opt.specifier == specifier then
          -- Set the value
          Archimedes.activeArcs[arc]:SetNew(value, max)
        end
      end
      end
    end
  end
end



------------------------------------
----------[[  CONTROLLER  ]]--------
----parses input, updated events----
--------(activates the model)-------
------------------------------------


function Archimedes.SaveVar(addon)
	if addon == "Archimedes" then
		if Archimedes.Options then
			Archimedes_Options = Archimedes.Options
		end
	end
end
function Archimedes.LoadVar(addon)
	if addon == "Archimedes" then
		if Archimedes_Options then
			Archimedes.Options = Archimedes_Options
		else
			Archimedes.Options = {}
		end
	end
end



function Archimedes.StupidHeadHack()
  Archimedes.ArcLoader()
  return Archimedes.startedOn + 15 > Inspect.Time.Real()
end


------------
---- Event hooks
-----------------------

-- Events for value updates
table.insert(Event.Unit.Detail.Health, {function (data) Archimedes.StatHandler(data, "health") end, "Archimedes", "Health refresh"})
table.insert(Event.Unit.Detail.HealthMax, {function (data) Archimedes.StatHandler(data, "health", "max") end, "Archimedes", "Health Max refresh"})
table.insert(Event.Unit.Detail.Mana, {function (data) Archimedes.StatHandler(data, "mana") end, "Archimedes", "Mana refresh"})
table.insert(Event.Unit.Detail.ManaMax, {function (data) Archimedes.StatHandler(data, "mana", "max") end, "Archimedes", "Mana Max refresh"})
table.insert(Event.Unit.Detail.Energy, {function (data) Archimedes.StatHandler(data, "energy") end, "Archimedes", "Energy refresh"})
table.insert(Event.Unit.Detail.EnergyMax, {function (data) Archimedes.StatHandler(data, "energy", "max") end, "Archimedes", "Energyh Max refresh"})
table.insert(Event.Unit.Detail.Power, {function (data) Archimedes.StatHandler(data, "power") end, "Archimedes", "Power refresh"})
table.insert(Event.Unit.Detail.Charge, {function (data) Archimedes.StatHandler(data, "charge") end, "Archimedes", "Charge refresh"})

-- Slash command
table.insert(Command.Slash.Register("arch"), {Archimedes.SlashHandler, "Archimedes", "Slash refresh"})

-- Saved variable stuff
table.insert(Event.Addon.SavedVariables.Save.Begin, {Archimedes.SaveVar, "Archimedes", "Save variables"})
table.insert(Event.Addon.SavedVariables.Load.End, {Archimedes.LoadVar, "Archimedes", "Load variables"})



table.insert(Event.System.Update.Begin, {function ()
  Archimedes.Tick()
end, "Archimedes", "Ticker"})



table.insert(Event.Addon.Startup.End, {function ()
  Archimedes.initialised = false
  Archimedes.startedOn = Inspect.Time.Real()
end, "Archimedes", "Not initialised"})


table.insert(Event.Unit.Available,  {function (unit)
  if not Archimedes.initialised then
    Archimedes.Init()
  end
end, "Archimedes", "Initialiser"})


table.insert(Event.Unit.Available,  {function (unit)
  if Archimedes.StupidHeadHack then
  if not Archimedes.StupidHeadHack() then
    Archimedes.StupidHeadHack = nil
  end
  end
end, "Archimedes", "unitadd"})