kAlert = {}

kAlert.context = UI.CreateContext("kAlert")

kAlert.version = tonumber(Inspect.Addon.Detail(Inspect.Addon.Current()).toc.Version)

kAlert.alertSet = {}
kAlert.alertSet.count = 0
kAlert.alertSet.alerts = {}
kAlert.alertSet.max = 15
kAlert.alertSubSet = {}
kAlert.alertSubSet.count = 0
kAlert.alertSubSet.alerts = {}

kAlert.screenObjects = {}
kAlert.screenObjects.object = nil
kAlert.screenObjects.objectCount = 0
kAlert.screenObjects.max = 20
kAlert.screenObjects.fixLayer = false

kAlert.units = {"player","player.target","focus","player.pet"}
kAlert.unitList = {}
kAlert.unitStateList = {}
kAlert.unitRelation = {friendly = 1,hostile = 2}
kAlert.combat = false
kAlert.abilityList = {}

kAlert.changeHandler = {}
kAlert.changeHandler.buffChanged = false
kAlert.changeHandler.abilityChanged = false
kAlert.changeHandler.resourceChanged = false
kAlert.changeHandler.castingChanged = false

kAlert.systemScanner = {}
kAlert.systemScanner.activeScanning = false
kAlert.systemScanner.nextScan = math.floor(Inspect.Time.Frame())

kAlert.debug = false

function logmsg(msg)
	if kAlert.debug then print(msg) end
end

function kAlert.main()

	table.insert(Event.System.Secure.Enter, {function () kAlert.combat = true end, "kAlert", "enterCombat"})
	table.insert(Event.System.Secure.Leave, {function () kAlert.combat = false end, "kAlert", "exitCombat"})

	table.insert(Event.Unit.Add, {kAlert.changeHandler.checkUnitChanged, "kAlert", "unitChangedAdd"})
	table.insert(Event.Unit.Remove, {kAlert.changeHandler.checkUnitChanged, "kAlert", "unitChangedRemove"})

	table.insert(Event.Unit.Castbar, {kAlert.changeHandler.checkCasting, "kAlert", "castDetected"})

	table.insert(Event.Buff.Add, {kAlert.changeHandler.checkBuffChanged, "kAlert", "buffAdded"})
	table.insert(Event.Buff.Remove, {kAlert.changeHandler.checkBuffChanged, "kAlert", "buffRemoved"})

	table.insert(Event.Ability.Cooldown.Begin, {kAlert.changeHandler.checkAbilityChanged, "kAlert", "cooldownBegin"})
	table.insert(Event.Ability.Cooldown.End, {kAlert.changeHandler.checkAbilityChanged, "kAlert", "cooldownEnd"})
	table.insert(Event.Ability.Usable.False, {kAlert.changeHandler.checkAbilityChanged, "kAlert", "cooldownEnd"})
	table.insert(Event.Ability.Usable.True, {kAlert.changeHandler.checkAbilityChanged, "kAlert", "cooldownEnd"})
	
	table.insert(Event.Unit.Detail.Health, {kAlert.changeHandler.checkResourceChanged, "kAlert", "healthChanged"})
	table.insert(Event.Unit.Detail.Mana, {kAlert.changeHandler.checkResourceChanged, "kAlert", "manaChanged"})
	table.insert(Event.Unit.Detail.Energy, {kAlert.changeHandler.checkResourceChanged, "kAlert", "energyChanged"})
	table.insert(Event.Unit.Detail.Power, {kAlert.changeHandler.checkResourceChanged, "kAlert", "powerChanged"})
	table.insert(Event.Unit.Detail.Charge, {kAlert.changeHandler.checkResourceChanged, "kAlert", "chargeChanged"})
	table.insert(Event.Unit.Detail.Combo, {kAlert.changeHandler.checkResourceChanged, "kAlert", "comboChanged"})

	table.insert(Event.System.Update.Begin, {kAlert.eventHandler, "kAlert", "eventHandler"})

end

function kAlert.alertSet.load()

	kAlert.alertSet.count = 0
	kAlert.alertSet.alerts = {}
	kAlert.alertSubSet.count = 0
	kAlert.alertSubSet.alerts = {}
	for id, details in pairs (kAlertAlerts) do
		if details.set == kAlertSet and kAlert.alertSet.count < kAlert.screenObjects.max then
			kAlert.alertSet.alerts[details.name] = details
			kAlert.alertSet.count = kAlert.alertSet.count + 1
		end
		if kAlertSubSet > 0 and details.set == kAlertSubSet + 5 and kAlert.alertSubSet.count < kAlert.screenObjects.max then
			kAlert.alertSubSet.alerts[details.name] = details
			kAlert.alertSubSet.count = kAlert.alertSubSet.count + 1
		end
	end

end

function kAlert.alertSet.save()

	for id, details in pairs (kAlertAlerts) do
		if details.set == kAlertSet then
			kAlertAlerts[id] = nil
		end
	end
	
	for id, details in pairs (kAlert.alertSet.alerts) do
		kAlertAlerts[tostring(kAlertSet) .. details.name] = details
	end
	
end

function kAlert.alertSet.add(alert)

	if kAlert.alertSet.count < kAlert.screenObjects.max and alert ~= nil then
		kAlert.alertSet.alerts[alert.name] = alert
		kAlert.alertSet.count = kAlert.alertSet.count + 1
		kAlert.alertSet.save()
		return true
	end
	
	return false
	
end

function kAlert.alertSet.delete(name)
	
	if kAlert.alertSet.alerts[name] ~= nil then
		kAlert.alertSet.alerts[name] = nil
		kAlert.alertSet.count = kAlert.alertSet.count - 1
		kAlert.alertSet.save()
		return true
	end
	
	return false
	
end

function kAlert.unitState(details)
	if details ~= nil then
		if kAlert.unitStateList[details.unit] ~= nil then
			return ((kAlert.unitStateList[details.unit].live or details.type ~= 1) and (details.unitRelation == 0 or details.unitRelation == kAlert.unitStateList[details.unit].relation))
		end
	end
	return false
end

function kAlert.commandHandler(commandline)
	
	if string.len(commandline) == 0 then
		kAlert.screenObjects.hide()
		kAlert.config.main()
	else
		local command = string.match(commandline,"%a+=%d+")
		if command ~= nil and string.match(command,"%a+") == "subset" then
			local setNumber = tonumber(string.match(command,"%d+"))
			if setNumber > 0 and setNumber < kAlert.alertSet.max - 4 then
				kAlertSubSet = setNumber
				kAlert.screenObjects.refresh()
			end
			print("Set=" .. tostring(kAlertSet) .. " Sub Set=" .. tostring(kAlertSubSet))
		elseif command ~= nil and string.match(command,"%a+") == "set" then
			local setNumber = tonumber(string.match(command,"%d+"))
			if setNumber > 0 and setNumber < 6 then
				kAlertSet = setNumber
				kAlert.screenObjects.refresh()
			end
			print("Set=" .. tostring(kAlertSet) .. " Sub Set=" .. tostring(kAlertSubSet))
		elseif string.match(commandline,"help") == "help" then
			print("HELP!!!!")
			if kAlert.config.helpFrame == nil then
				kAlert.config.buildHelpFrame()
			end
			kAlert.config.helpFrame:SetVisible(true)
		elseif string.match(commandline,"debug") == "debug" then
			kAlert.debug = not kAlert.debug
			if kAlert.debug then
				print("Debug ON")
			else
				print("Debug OFF")
			end
		else
			print("Karuul Alert Commands")
			print("Usage: /KaruulAlert {command}")
			print("Commands:")
			print("set={number} - to change current alert set")
			print("subset={number} - to change current alert sub set")
			print("help - to display addon help file")
		end
	end
	
end

table.insert(Command.Slash.Register("KAlert"), {kAlert.commandHandler, "kAlert", "config"})
table.insert(Command.Slash.Register("KaruulAlert"), {kAlert.commandHandler, "kAlert", "config"})

function kAlert.settingsHandler() 

	if kAlertVersion == nil then
		kAlertVersion = 1.00
	end
	
	if kAlertGlobalVersion == nil then
		kAlertGlobalVersion = 1.00
	end
	
	if kAlertSet == nil then
		kAlertSet = 1
	end
	
	if kAlertSubSet == nil then
		kAlertSubSet = 0
	end
	
	if kAlertAlerts == nil then
		kAlertAlerts = {}
	end
	
	if kAlertGlobalItems == nil then
		kAlertGlobalItems = {}
	end
	
	kAlert.screenObjects.init()
	
end

table.insert(Event.Addon.SavedVariables.Load.End, {kAlert.settingsHandler, "kAlert", "vaiablesLoaded"})

function kAlert.eventHandler()
	
	if kAlert.screenObjects.fixLayer then kAlert.screenObjects.updateLayers() end
	
	local scanTime = Inspect.Time.Frame()
	if kAlert.systemScanner.nextScan < scanTime then
		kAlert.systemScanner.nextScan = scanTime + 0.2

		if not kAlert.config.active then
			for id, details in pairs (kAlert.screenObjects.object) do
				if details.duration > 0 then
					local remaining = details.duration - (scanTime - details.begin)
					details.setTimer(remaining)
					if details.type == 1 then -- Buff Timer
						if remaining < details.timerLength and details.timer and not details.typeToggle then details:SetVisible(true) end
					elseif details.type == 2 then -- Ability Timer
						if remaining < details.timerLength and details.timer and details.typeToggle then details:SetVisible(true) end
					end
				end
			end
			if kAlert.changeHandler.buffChanged then kAlert.processBuffs() end
			if kAlert.changeHandler.abilityChanged then kAlert.processAbilities() end
			if kAlert.changeHandler.resourceChanged then kAlert.processResources() end
			if kAlert.changeHandler.castingChanged then kAlert.processCasting() end
		end
		
		if kAlert.systemScanner.activeScanning then
			kAlert.systemScanner.scan()
		end
		
	end
	
end

function kAlert.updateAlerts()
	for id, details in pairs(kAlertAlerts) do
		if details.layer == nil then details.layer = 1 end
		if details.combatOnly == nil then details.combatOnly = false end
		if details.itemValue == nil then details.itemValue = 1 end
		if details.itemLength == nil then details.itemLength = 0 end
		if details.type == 1 and details.imageSource == "Rift" then
			local itemNames = kUtils.split(",",details.itemName)
			if table.getn(itemNames) > 1 then
				if kAlertGlobalItems[itemNames[1]] ~= nil then
					details.image = kAlertGlobalItems[itemNames[1]].icon
				end
			end
		end
		if details.imageOpacity == nil then details.imageOpacity = 1 end
		if details.text == nil then details.text = "" end
		if details.textOpacity == nil then details.textOpacity = 1 end
		if details.textFont == nil then
			if details.timerFont ~= nil then
				details.textFont = details.timerFont
				details.timerFont = nil
			else
				details.textFont = ""
			end
		end
		if details.textRed == nil then details.textRed = 1 end
		if details.textGreen == nil then details.textGreen = 1 end
		if details.textBlue == nil then details.textBlue = 1 end
		if details.textSize == nil then details.textSize = 30 end
		if details.textLocation == nil then details.textLocation = "CENTER" end
		if details.textInside == nil then details.textInside = true end
		if details.timerLength == nil then details.timerLength = 5 end
		if details.selfCast == nil then details.selfCast = false end
		if details.range == nil then details.range = false end
		if details.rangeLow == nil then details.rangeLow = 0 end
		if details.rangeHigh == nil then details.rangeHigh = 0 end
		if details.unitRelation == nil then details.unitRelation = 0 end
	end
end

function kAlert.updateGlobalItems()
	for id, details in pairs(kAlertGlobalItems) do
		if id ~= details.name then
			kAlertGlobalItems[details.name] = details
			kAlertGlobalItems[id] = nil
		end
		if details.update == nil then details.update = true end
		if details.icon == nil then kAlertGlobalItems[details.name] = nil end
	end
end

function kAlert.screenObjects.init()

	if kAlert.screenObjects.object == nil then
		if kAlertVersion < kAlert.version then
			print("v" .. tostring(kAlertVersion) .. " -> v" .. tostring(kAlert.version))
			kAlertVersion = kAlert.version
		end
		kAlert.updateAlerts()
		kAlert.updateGlobalItems()
		print("v" .. tostring(kAlert.version))
		kAlert.screenObjects.object = {}
		for i = 1, kAlert.screenObjects.max * 2, 1 do
			kAlert.screenObjects.object[i] = kAlert.screenObjects.addObject(i)
		end
		kAlert.screenObjects.refresh()
	end
	
end

function kAlert.screenObjects.addObject(id)

	local iID = tostring(id)
	local object = UI.CreateFrame("Frame", "objectFrame" .. iID, kAlert.context)
	object:SetWidth(75)
	object:SetHeight(75)
	object:SetPoint ("TOPLEFT", kAlert.context, "TOPLEFT")
	object:SetVisible(false)
	object:SetBackgroundColor(0, 0, 0, 0)
	object:SetLayer(-1)

	object.name = nil
	object.unit = nil
	object.unitRelation = 0
	object.type = nil
	object.typeToggle = nil
	object.range = false
	object.rangeLow = 0
	object.rangeHigh = 0
	object.itemId = nil
	object.itemName = nil
	object.itemValue = nil
	object.itemValuePercent = nil
	object.itemLength = 0
	object.timer = nil
	object.timerLength = 0
	object.combatOnly = nil
	object.begin = 0
	object.duration = 0
	object.selfCast = false
		
	object.image = UI.CreateFrame("Texture", "objectIcon" .. iID, object)
	object.image:SetPoint("CENTER", object, "CENTER", 0, 2)
	object.image:SetLayer(0)
	
	object.text = UI.CreateFrame("Text", "objectText" .. iID, object)
	object.text:SetText('')
	object.text:SetFontSize(object:GetWidth()/2)
	object.text:SetFontColor(1, 1, 1, 1)
	object.text:SetLayer(1)
	object.text:SetPoint("CENTER", object.image, "CENTER")
	
	object.counter = UI.CreateFrame("Text", "objectDuration" .. iID, object)
	object.counter:SetText('')
	object.counter:SetFontSize(object:GetWidth()/2)	
	object.counter:SetFontColor(1, 1, 1, 1)
	object.counter:SetLayer(2)
	object.counter:SetPoint("CENTER", object.image, "CENTER")
	object.counter:SetVisible(false)
	
	function object.setDims(width, height)
		object:SetWidth(width)
		object:SetHeight(height)
		object.image:SetWidth(width)
		object.image:SetHeight(height)
	end
	
	function object.setTimer(value)
		if value >= 0 then
			object.counter:SetText(tostring(math.floor(value)))
		else
			object.counter:SetText("")
		end
	end
	
	function object.setText(value)
		object.text:SetText(value)
	end
	
	return object
	
end

function kAlert.screenObjects.clear()
	for i = 1, kAlert.screenObjects.max * 2, 1 do
		kAlert.screenObjects.object[i]:SetVisible(false)
		kAlert.screenObjects.object[i]:SetLayer(1)
		kAlert.screenObjects.object[i].name = nil
		kAlert.screenObjects.object[i].unit = nil
		kAlert.screenObjects.object[i].unitRelation = 0
		kAlert.screenObjects.object[i].type = nil
		kAlert.screenObjects.object[i].typeToggle = nil
		kAlert.screenObjects.object[i].itemId = nil
		kAlert.screenObjects.object[i].itemName = nil
		kAlert.screenObjects.object[i].itemValue = nil
		kAlert.screenObjects.object[i].itemValuePercent = nil
		kAlert.screenObjects.object[i].itemLength = 0
		kAlert.screenObjects.object[i].setDims(10,10)
		kAlert.screenObjects.object[i].timer = nil
		kAlert.screenObjects.object[i].timerLength = 0
		kAlert.screenObjects.object[i].combatOnly = nil
		kAlert.screenObjects.object[i].begin = 0
		kAlert.screenObjects.object[i].duration = 0
		kAlert.screenObjects.object[i].setTimer(-1)
		kAlert.screenObjects.object[i].setText("")
		kAlert.screenObjects.object[i].selfCast = false
	end
end

function kAlert.screenObjects.add(alert)
	if alert.active == true and kAlert.screenObjects.objectCount < kAlert.screenObjects.max * 2 then
		kAlert.screenObjects.objectCount = kAlert.screenObjects.objectCount + 1
		local screenObject = kAlert.screenObjects.object[kAlert.screenObjects.objectCount]
		-- Check Data
		screenObject.name = alert.name
		screenObject:SetLayer(alert.layer)
		screenObject.unit = alert.unit
		screenObject.unitRelation = alert.unitRelation
		screenObject.type = alert.type
		screenObject.typeToggle = alert.typeToggle
		screenObject.range = alert.range
		screenObject.rangeLow = alert.rangeLow
		screenObject.rangeHigh = alert.rangeHigh
		screenObject.itemId = alert.itemId
		screenObject.itemLength = alert.itemLength
		screenObject.selfCast = alert.selfCast
		if alert.type == 1 then
			screenObject.itemName = kUtils.split(",",alert.itemName)
			if screenObject.itemLength ~= 0 then
				for id, itemName in pairs(screenObject.itemName) do
					screenObject.itemName[id] = itemName .. tostring(screenObject.itemLength)
				end
			end
			if screenObject.selfCast then
				for id, itemName in pairs(screenObject.itemName) do
					screenObject.itemName[id] = itemName .. "S"
				end
			end
		elseif alert.type == 2 then
			if kAlertGlobalItems[alert.itemName] ~= nil then kAlert.abilityList[kAlertGlobalItems[alert.itemName].ability] = alert.itemName end
			screenObject.itemName = alert.itemName
		else
			screenObject.itemName = alert.itemName
		end
		screenObject.itemValue = alert.itemValue
		screenObject.itemValuePercent = alert.itemValuePercent
		screenObject.combatOnly = alert.combatOnly
		screenObject.timerLength = alert.timerLength
		-- Image
		screenObject.setDims(alert.imageWidth,alert.imageHeight)
		screenObject.image:SetTexture(alert.imageSource, alert.image)
		screenObject.image:SetAlpha(alert.imageOpacity)
		screenObject:SetPoint("TOPLEFT", kAlert.context, "TOPLEFT", alert.imageX, alert.imageY)
		-- Text
		screenObject.text:SetText(alert.text)
		screenObject.text:ClearAll()
		screenObject.text:SetFontColor(alert.textRed,alert.textGreen,alert.textBlue)
		screenObject.text:SetAlpha(alert.textOpacity)
		if string.len(alert.textFont) > 0 then
			screenObject.text:SetFont("kAlert",alert.textFont)
		else
			screenObject.text:SetFont("Rift","$Flareserif_medium")
		end
		screenObject.text:SetFontSize(alert.textSize)
		screenObject.text:SetHeight(alert.textSize + 5)
		if alert.textLocation == "CENTER" then
			screenObject.text:SetPoint("CENTER", screenObject.image, "CENTER")
		elseif alert.textLocation == "TOPCENTER" then
			if alert.textInside then
				screenObject.text:SetPoint("TOPCENTER", screenObject.image, "TOPCENTER")
			else
				screenObject.text:SetPoint("BOTTOMCENTER", screenObject.image, "TOPCENTER")
			end
		elseif alert.textLocation == "BOTTOMCENTER" then
			if alert.textInside then
				screenObject.text:SetPoint("BOTTOMCENTER", screenObject.image, "BOTTOMCENTER")
			else
				screenObject.text:SetPoint("TOPCENTER", screenObject.image, "BOTTOMCENTER")
			end
		elseif alert.textLocation == "LEFTCENTER" then
			if alert.textInside then
				screenObject.text:SetPoint("LEFTCENTER", screenObject.image, "LEFTCENTER")
			else
				screenObject.text:SetPoint("RIGHTCENTER", screenObject.image, "LEFTCENTER")
			end
		elseif alert.textLocation == "RIGHTCENTER" then
			if alert.textInside then
				screenObject.text:SetPoint("RIGHTCENTER", screenObject.image, "RIGHTCENTER")
			else
				screenObject.text:SetPoint("LEFTCENTER", screenObject.image, "RIGHTCENTER")
			end
		end
		-- Timer
		screenObject.timer = alert.timer
		screenObject.counter:SetVisible(alert.timer)
		screenObject.counter:ClearAll()
		if string.len(alert.textFont) > 0 then
			screenObject.counter:SetFont("kAlert",alert.textFont)
		else
			screenObject.counter:SetFont("Rift","$Flareserif_medium")
		end
		screenObject.counter:SetFontSize(alert.timerSize)
		screenObject.counter:SetHeight(alert.timerSize + 5)
		screenObject.counter:SetFontColor(alert.textRed,alert.textGreen,alert.textBlue)
		screenObject.counter:SetAlpha(alert.textOpacity)
		if alert.timerLocation == "CENTER" then
			screenObject.counter:SetPoint("CENTER", screenObject.image, "CENTER")
		elseif alert.timerLocation == "TOPCENTER" then
			if alert.timerInside then
				screenObject.counter:SetPoint("TOPCENTER", screenObject.image, "TOPCENTER")
			else
				screenObject.counter:SetPoint("BOTTOMCENTER", screenObject.image, "TOPCENTER")
			end
		elseif alert.timerLocation == "BOTTOMCENTER" then
			if alert.timerInside then
				screenObject.counter:SetPoint("BOTTOMCENTER", screenObject.image, "BOTTOMCENTER")
			else
				screenObject.counter:SetPoint("TOPCENTER", screenObject.image, "BOTTOMCENTER")
			end
		elseif alert.timerLocation == "LEFTCENTER" then
			if alert.timerInside then
				screenObject.counter:SetPoint("LEFTCENTER", screenObject.image, "LEFTCENTER")
			else
				screenObject.counter:SetPoint("RIGHTCENTER", screenObject.image, "LEFTCENTER")
			end
		elseif alert.timerLocation == "RIGHTCENTER" then
			if alert.timerInside then
				screenObject.counter:SetPoint("RIGHTCENTER", screenObject.image, "RIGHTCENTER")
			else
				screenObject.counter:SetPoint("LEFTCENTER", screenObject.image, "RIGHTCENTER")
			end
		end
		screenObject:SetVisible(false)
	end
end

function kAlert.screenObjects.updateLayers()
	for i = 1, kAlert.screenObjects.max * 2, 1 do
		kAlert.screenObjects.object[i]:SetLayer(kAlert.screenObjects.object[i]:GetLayer())
	end
	kAlert.screenObjects.fixLayer = false
end

function kAlert.screenObjects.refresh()

	kAlert.screenObjects.objectCount = 0
	kAlert.screenObjects.clear()
	kAlert.alertSet.load()
	local abilityList = Inspect.Ability.List()
	if abilityList ~= nil then
		local abilityDetails = Inspect.Ability.Detail(abilityList)
		for id, details in pairs(abilityDetails) do
			if details.name ~= nil then
				if kAlertGlobalItems[details.name] == nil or kAlertGlobalItems[details.name].update  or kAlertGlobalItems[details.name].ability ~= id then
					kAlertGlobalItems[details.name] = {ability = id, name = details.name, icon = details.icon, update = false}
					logmsg(kAlertTexts.msgNewAbility .. ': ' .. details.name) -- .. '-' .. id)
				end
			end
		end
	end
	for id, details in pairs (kAlert.alertSet.alerts) do
		kAlert.screenObjects.add(details)
	end
	for id, details in pairs (kAlert.alertSubSet.alerts) do
		kAlert.screenObjects.add(details)
	end
	kAlert.processBuffs()
	kAlert.processAbilities()
	kAlert.processResources()
	kAlert.screenObjects.fixLayer = true
end

function kAlert.screenObjects.hide()
	for id, details in pairs (kAlert.screenObjects.object) do
		details:SetVisible(false)
	end
end

function kAlert.changeHandler.checkCasting(units)

	if not kAlert.config.active and not kAlert.changeHandler.castingChanged then
		local unitList = Inspect.Unit.List()
		for id, details in pairs(units) do
			for i = 1, table.getn(kAlert.units), 1 do
				if unitList[id] == kAlert.units[i] then kAlert.changeHandler.castingChanged = true end
			end
		end
	end

end

function kAlert.processCasting()

	local unitsResources = {}
	for i = 1, table.getn(kAlert.units), 1 do
		local spellCast = Inspect.Unit.Castbar(kAlert.units[i])
		if spellCast ~= nil then
			unitsResources[kAlert.units[i]] = spellCast.abilityName
		end
	end
	for id, details in pairs (kAlert.screenObjects.object) do
		if details.type ~= nil then
			if details.type == 4 then -- Casting
				local showObject = false
				if not details.combatOnly or (details.combatOnly and kAlert.combat) then
					if unitsResources[details.unit] ~= nil and kAlert.unitState(details) then
						if string.len(details.itemName) == 0 or unitsResources[details.unit] == details.itemName then
							showObject = true
						end
					end
				end
				details:SetVisible(showObject)
			end
		end
	end
	kAlert.changeHandler.castingChanged = false

end

function kAlert.changeHandler.checkUnitChanged(units)
	for id, details in pairs(units) do
		for i = 1, table.getn(kAlert.units), 1 do
			if details == kAlert.units[i] or (kAlert.unitList[id] ~= nil and not details) then
				if details then 
					kAlert.unitList[id] = details
				else
					kAlert.unitList[id] = nil
				end
				kAlert.changeHandler.buffChanged = true
				kAlert.changeHandler.resourceChanged = true
			end
		end
	end
	for i = 1, table.getn(kAlert.units), 1 do
		local unit = Inspect.Unit.Detail(kAlert.units[i])
		if unit ~= nil then
			local relation = kAlert.unitRelation[unit.relation]
			if relation == nil then relation = 0 end
			kAlert.unitStateList[kAlert.units[i]] = {relation = relation, live = (unit.health and unit.health > 0)}
		else
			kAlert.unitStateList[kAlert.units[i]] = nil
		end
	end
end

function kAlert.changeHandler.checkBuffChanged(unit)

	if not kAlert.config.active and not kAlert.changeHandler.buffChanged then
		local unitList = Inspect.Unit.List()
		for i = 1, table.getn(kAlert.units), 1 do
			if unitList[unit] == kAlert.units[i] then kAlert.changeHandler.buffChanged = true end
		end
	end
	
end

function kAlert.processBuffs()

	local unitsBuffs = {}
	for i = 1, table.getn(kAlert.units), 1 do
		local unitBuffs = Inspect.Buff.List(kAlert.units[i])
		if unitBuffs ~= nil then
			local buffList = {}
			local buffDetails = Inspect.Buff.Detail(kAlert.units[i], unitBuffs)
			for id, details in pairs(buffDetails) do
				if details.name ~= nil then
					if kAlertGlobalItems[details.name] == nil or kAlertGlobalItems[details.name].update then
						kAlertGlobalItems[details.name] = {ability = details.ability, name = details.name, icon = details.icon, update = false}
						logmsg(kAlertTexts.msgNewBuff .. ': ' .. details.name) -- .. '-' .. details.ability)
					end
					local buffStacks = 1
					if details.stack ~= nil then buffStacks = details.stack end
					if details.remaining == nil then
						buffList[details.name] = {remaining = 0, begin = 0, duration = 0, stacks = buffStacks}
						buffList[details.name .. "0"] = {remaining = 0, begin = 0, duration = 0, stacks = buffStacks}
						if kAlert.unitList[details.caster] == "player" then
							buffList[details.name .. "S"] = {remaining = 0, begin = 0, duration = 0, stacks = buffStacks}
							buffList[details.name .. "0S"] = {remaining = 0, begin = 0, duration = 0, stacks = buffStacks}
						end
					else
						buffList[details.name] = {remaining = details.remaining, begin = details.begin, duration = details.duration, stacks = buffStacks}
						buffList[details.name .. tostring(math.floor(details.duration + 0.5))] = {remaining = details.remaining, begin = details.begin, duration = details.duration, stacks = buffStacks}
						if kAlert.unitList[details.caster] == "player" then
							buffList[details.name .. "S"] = {remaining = details.remaining, begin = details.begin, duration = details.duration, stacks = buffStacks}
							buffList[details.name .. tostring(math.floor(details.duration + 0.5)) .. "S"] = {remaining = details.remaining, begin = details.begin, duration = details.duration, stacks = buffStacks}
						end
					end
				end
			end
			unitsBuffs[kAlert.units[i]] = buffList
		end
	end
	
	for id, details in pairs (kAlert.screenObjects.object) do
		if details.type ~= nil then
			if details.type == 1 then -- Buff
				local showObject = false
				details.setTimer(-1)
				details.begin = 0
				details.duration = 0
				if not details.combatOnly or (details.combatOnly and kAlert.combat) then
					if unitsBuffs[details.unit] ~= nil and kAlert.unitState(details) then
						local buffDetails = nil
						for id, itemName in pairs(details.itemName) do
							if unitsBuffs[details.unit][itemName] ~= nil then
								buffDetails = unitsBuffs[details.unit][itemName]
							end
						end
						local timeLeft = details.timerLength
						if buffDetails ~= nil and buffDetails.stacks >= details.itemValue then
							timeLeft = buffDetails.remaining
							details.setTimer(timeLeft)
							details.begin = buffDetails.begin
							details.duration = buffDetails.duration
						end
						if timeLeft < details.timerLength and details.timer and not details.typeToggle then
							showObject = true
						else
							showObject = (((buffDetails ~= nil) and (buffDetails.stacks >= details.itemValue)) == details.typeToggle)
						end
					end
				end
				details:SetVisible(showObject)
			end
		end
	end

	kAlert.changeHandler.buffChanged = false

end

function kAlert.changeHandler.checkAbilityChanged()

	if not kAlert.config.active and not kAlert.changeHandler.abilityChanged then
		kAlert.changeHandler.abilityChanged = true
	end
	
end

function kAlert.processAbilities()

	if kAlert.config.active then return end
	
	local activeAbilities = {}
	if kAlert.abilityList ~= nil then
		local abilityDetails = Inspect.Ability.Detail(kAlert.abilityList)
		for id, details in pairs(abilityDetails) do
			if details.name ~= nil then
				if details.currentCooldownDuration ~= nil then
					if details.currentCooldownDuration > 2 and details.currentCooldownRemaining ~= nil then
						activeAbilities[details.name] =
						{
							remaining = details.currentCooldownRemaining, 
							begin = details.currentCooldownBegin, 
							duration = details.currentCooldownDuration
						}
					elseif details.unusable then
						activeAbilities[details.name] =
						{
							remaining = -1, 
							begin = -1, 
							duration = -1
						}
					end
				elseif details.unusable then
					activeAbilities[details.name] =
					{
						remaining = -1, 
						begin = -1, 
						duration = -1
					}
				end
			end
		end
	end

	for id, details in pairs (kAlert.screenObjects.object) do
		if details.type ~= nil then
			if details.type == 2 then -- Ability
				local showObject = false
				details.setTimer(-1)
				details.begin = 0
				details.duration = 0
				if not details.combatOnly or (details.combatOnly and kAlert.combat) then
					local abilityDetails = activeAbilities[details.itemName]
					local timeLeft = details.timerLength
					if abilityDetails ~= nil then
						timeLeft = abilityDetails.remaining
						details.setTimer(timeLeft)
						details.begin = abilityDetails.begin
						details.duration = abilityDetails.duration
					end
					if timeLeft < details.timerLength and details.timer and details.typeToggle then
						showObject = true
					else
						showObject = ((abilityDetails == nil) == details.typeToggle)
					end
				end
				details:SetVisible(showObject)
			end
		end
	end
	
	kAlert.changeHandler.abilityChanged = false
	
end

function kAlert.changeHandler.checkResourceChanged(units)

	if not kAlert.config.active and not kAlert.changeHandler.resourceChanged then
		local unitList = Inspect.Unit.List()
		for id, details in pairs(units) do
			for i = 1, table.getn(kAlert.units), 1 do
				if unitList[id] == kAlert.units[i] then kAlert.changeHandler.resourceChanged = true end
			end
		end
	end

end

function kAlert.processResources()

	if kAlert.config.active then return end
	
	local unitsResources = {}
	for i = 1, table.getn(kAlert.units), 1 do
		local unitResources =  Inspect.Unit.Detail(kAlert.units[i])
		local resourceList = {}
		if unitResources ~= nil then
			if unitResources.health then
				resourceList[1] = {value = unitResources.health, percent = (unitResources.health/unitResources.healthMax)*100}
			end
			if unitResources.mana then
				resourceList[2] = {value = unitResources.mana, percent = (unitResources.mana/unitResources.manaMax)*100}
			end
			if unitResources.energy then
				resourceList[3] = {value = unitResources.energy, percent = (unitResources.energy/unitResources.energyMax)*100}
			end
			if unitResources.power then
				resourceList[4] = {value = unitResources.power, percent = unitResources.power}
			end
			if unitResources.charge then
				resourceList[5] = {value = unitResources.charge, percent = unitResources.charge}
			end
			if unitResources.combo then
				if unitResources.comboUnit then
					if kAlert.unitList[unitResources.comboUnit] == "player.target" then
						resourceList[6] = {value = unitResources.combo, percent = unitResources.combo}
					end
				else
					resourceList[6] = {value = unitResources.combo, percent = unitResources.combo}
				end
			end
		end
		unitsResources[kAlert.units[i]] = resourceList
	end
	
	for id, details in pairs (kAlert.screenObjects.object) do
		if details.type ~= nil then
			if details.type == 3 then -- Resource
				local showObject = false
				if not details.combatOnly or (details.combatOnly and kAlert.combat) then
					if unitsResources[details.unit] ~= nil and kAlert.unitState(details) then
						if unitsResources[details.unit][details.itemId] ~= nil then
							local checkValue = 0
							if details.itemValuePercent then
								checkValue = unitsResources[details.unit][details.itemId].percent
							else
								checkValue = unitsResources[details.unit][details.itemId].value
							end
							if details.range then
								showObject = (checkValue >= details.rangeLow and checkValue <= details.rangeHigh)
							else
								showObject = ((checkValue >= details.itemValue) == details.typeToggle)
							end
						end
					end
				end
				details:SetVisible(showObject)
			end
		end
	end
	
	kAlert.changeHandler.resourceChanged = false

end

function kAlert.systemScanner.scan()

	local unitsBuffs = {}
	for i = 1, table.getn(kAlert.units), 1 do
		local unitBuffs = Inspect.Buff.List(kAlert.units[i])
		if unitBuffs ~= nil then
			local buffList = {}
			if unitBuffs ~= nil then
				local buffDetails = Inspect.Buff.Detail(kAlert.units[i], unitBuffs)
				for id, details in pairs(buffDetails) do
					if details.name ~= nil then
						if kAlertGlobalItems[details.name] == nil then
							kAlertGlobalItems[details.name] = {ability = details.ability, name = details.name, icon = details.icon, update = false}
							print(kAlertTexts.msgNewBuff .. ': ' .. details.name) -- .. '-' .. details.ability)
						end
					end
				end
			end
			unitsBuffs[kAlert.units[i]] = buffList
		end
	end
	
	local abilityList = Inspect.Ability.List()
	if abilityList then
		local abilityDetails = Inspect.Ability.Detail(abilityList)
		for id, details in pairs(abilityDetails) do
			if details.name ~= nil then
				if kAlertGlobalItems[details.name] == nil then
					kAlertGlobalItems[details.name] = {ability = id, name = details.name, icon = details.icon, update = false}
					print(kAlertTexts.msgNewAbility .. ': ' .. details.name) -- .. '-' .. id)
				end
			end
		end	
	end

end

kAlert.main()
