kUtils = {}

function kUtils.pairsByKeys(t, f)
	local a = {}
	for n in pairs(t) do table.insert(a, n) end
	table.sort(a, f)
	local i = 0      -- iterator variable
	local iter = function ()   -- iterator function
		i = i + 1
		if a[i] == nil then return nil
		else return a[i], t[a[i]]
		end
	end
	return iter
end

function kUtils.split(delimiter, text)
  local list = {}
  local pos = 1
  if string.find("", delimiter, 1) then -- this would result in endless loops
    return nil
  end
  while 1 do
    local first, last = string.find(text, delimiter, pos)
    if first then -- found?
      table.insert(list, string.sub(text, pos, first-1))
      pos = last+1
    else
      table.insert(list, string.sub(text, pos))
      break
    end
  end
  return list
end

function kUtils.join(delimiter, list)
  local len = getn(list)
  if len == 0 then 
    return "" 
  end
  local text = list[1]
  for i = 2, len do 
    text = text .. delimiter .. list[i] 
  end
  return text
end

function kUtils.booltostring(value)
	if value then
		return "T"
	else
		return "F"
	end
end

function kUtils.stringtobool(value)
	return (value == "T")
end

function kUtils.buildFrame(name, parent, height, width)

	local frame = UI.CreateFrame("Frame", name, parent)
	frame:SetHeight(height)
	frame:SetWidth(width)
	
	return frame
	
end

function kUtils.buildLabel(name, parent, height, width, text, wrap)

	if wrap == nil then wrap = false end
	local label = UI.CreateFrame("Text", name, parent)
	label:SetWordwrap(wrap)
	label:SetText(text)
	label:SetFontSize(height)
	label:SetWidth(width)
	label:SetFontColor(1, 1, 1, 1)
	
	function label.setWrapText(text)
		label:SetWordwrap(true)
		label:SetText(text)
	end
	
	return label
	
end

function kUtils.buildSlider(name, parent, height, width, minVal, maxVal)

	local frame = UI.CreateFrame("RiftSlider", name, parent)
	frame:SetHeight(height)
	frame:SetWidth(width)
	frame:SetRange(minVal, maxVal)
	
	return frame
	
end

function kUtils.buildCheckBox(name, parent, height, width, label, reverse)

	local checkBox = UI.CreateFrame("RiftCheckbox", name, parent)
	checkBox:SetHeight(height)
	checkBox:SetWidth(height)
	checkBox.label = kUtils.buildLabel(name .. "Label", checkBox, height-5, width - height - 3, label)
	if reverse then
		checkBox.label:SetPoint("CENTERRIGHT", checkBox, "CENTERLEFT", 3, 0)
	else
		checkBox.label:SetPoint("CENTERLEFT", checkBox, "CENTERRIGHT", 3, 0)
	end
	
	return checkBox
	
end

function kUtils.buildButton(name, parent, height, width, label, skin)

	local button = UI.CreateFrame("RiftButton", name, parent)
	if skin == "close" then
		button:SetSkin("close")
		return button
	else
		button:SetText(label)
		button:SetHeight(height)
		button:SetWidth(width)
	end
	
	return button
	
end

function kUtils.buildEditBox(name, parent, height, width, label)

	local editBox = UI.CreateFrame("Frame", name, parent)
	editBox:SetPoint("TOPLEFT", parent, "TOPLEFT")
	editBox:SetHeight(height)
	editBox:SetWidth(width)
	editBox.label = kUtils.buildLabel(name .. "Label", editBox, height - 5, width, label)
	editBox.label:ClearWidth()
	editBox.label:SetPoint("CENTERLEFT", editBox, "CENTERLEFT")
	editBox.label:SetLayer(1)
	editBox.text = UI.CreateFrame("RiftTextfield", name .. "Text", editBox)
	editBox.text:SetPoint("CENTERLEFT", editBox.label, "CENTERRIGHT")
	editBox.text:SetHeight (height)
	editBox.text:SetWidth (width - editBox.label:GetWidth())
	editBox.text:SetBackgroundColor(.25, .25, .25)
	editBox.text:SetLayer(1)
	editBox.outline = kUtils.setOutline(editBox.text)
	
	return editBox
	
end

function kUtils.buildImageBox(name, parent, height, width, source, image)

	local imageBox = UI.CreateFrame("Texture", name, parent)
	imageBox:SetWidth(width)
	imageBox:SetHeight(height)
	imageBox:SetBackgroundColor(0, 0, 0, 0.5)
	imageBox:SetTexture(source, image)
	imageBox:SetLayer(0)
	imageBox.outline = kUtils.setOutline(imageBox)
	
	function imageBox.setImage(source, image)
		imageBox.source = source
		imageBox.image = image
		imageBox:SetTexture(source, image)
	end
	
	return imageBox

end

function kUtils.buildToggleBox(name, parent, height, width, options, columns, allowNone)

	local toggleBox = UI.CreateFrame("Frame", name, parent)
	toggleBox:SetPoint("TOPLEFT", parent, "TOPLEFT", 30, 65)
	if columns == nil then columns = 1 end
	toggleBox:SetHeight(height * math.ceil(table.getn(options)/columns))
	toggleBox:SetWidth(width * columns)
	toggleBox:SetBackgroundColor(0, 0, 0, 0.25)
	toggleBox:SetLayer(-1)
	toggleBox.updateOptions = true
	toggleBox.cbOption = {}
	toggleBox.outline = kUtils.setOutline(toggleBox)
	toggleBox.allowNone = allowNone
	
	local row = 1
	for id, details in pairs (options) do
		option = kUtils.buildCheckBox(name .. "Option" .. tostring(id),toggleBox,height,width,details)
		if id == 1 then
			option:SetPoint("TOPLEFT", toggleBox, "TOPLEFT",1,0)
			option:SetChecked(true)
		elseif math.ceil(id/columns)-row == 0 then
			option:SetPoint("TOPLEFT", toggleBox.cbOption[id-1].label, "TOPRIGHT")
		else
			option:SetPoint("TOPLEFT", toggleBox.cbOption[id-columns], "BOTTOMLEFT")
			row = row + 1
		end
		if allowNone then
			function option.Event:CheckboxChange()
				if toggleBox.updateOptions then
					toggleBox.updateOptions = false
					local setValue = self:GetChecked()
					for i = 1, table.getn(options), 1 do
						toggleBox.cbOption[i]:SetChecked(false)
					end
					self:SetChecked(setValue)
					toggleBox.updateOptions = true
				end
				if toggleBox.change ~= nil then toggleBox.change() end
			end
		else
			function option.Event:CheckboxChange()
				if toggleBox.updateOptions then
					toggleBox.updateOptions = false
					for i = 1, table.getn(options), 1 do
						toggleBox.cbOption[i]:SetChecked(false)
					end
					self:SetChecked(true)
					toggleBox.updateOptions = true
				end
				if toggleBox.change ~= nil then toggleBox.change() end
			end
		end
		toggleBox.cbOption[id] = option
	end
	
	function toggleBox.setOptions(options)
		for i = 1, table.getn(toggleBox.cbOption), 1 do
			if options[i] ~= nil then toggleBox.cbOption[i].label:SetText(options[i]) end
		end
	end
	
	function toggleBox.getSelected()
		for i = 1, table.getn(options), 1 do
			if toggleBox.cbOption[i]:GetChecked() then return i end
		end
		return 0
	end
	
	function toggleBox.setSelected(id)
		if id == 0 or id == nil then
			toggleBox.cbOption[1]:SetChecked(true)
			if toggleBox.allowNone then toggleBox.cbOption[1]:SetChecked(false) end
		else
			toggleBox.cbOption[id]:SetChecked(true)
		end
	end
	
	return toggleBox
	
end

function kUtils.buildListPane(name, parent, height, width, size)

	local listPane = UI.CreateFrame("Frame", name, parent)
	listPane:SetHeight (height)
	listPane:SetWidth (width)
	listPane:SetBackgroundColor(0, 0, 0, 0.5)
	listPane:SetLayer(-1)
	local itemHight = listPane:GetHeight() / size
	listPane.itemMax = size
	listPane.lpItem = {}
	listPane.outline = kUtils.setOutline(listPane)
	
	for i = 1, size, 1 do
		item = UI.CreateFrame("Text", name .. "Item" .. tostring(i), listPane)
		item:SetText("")
		item:SetFontSize(itemHight - 5)
		item:SetHeight(itemHight)
		item:SetWidth(listPane:GetWidth())
		item:SetFontColor(1, 1, 1, 1)
		item.selected = false
		item.enabled = true
		
		if i == 1 then
			item:SetPoint("TOPLEFT", listPane, "TOPLEFT")
		else
			item:SetPoint("TOPLEFT", listPane.lpItem[i-1], "BOTTOMLEFT")
		end
		
		function item.Event:MouseIn()
			if listPane.onMouseOver ~= nil then listPane.onMouseOver() end
		end
		
		function item.Event:LeftClick()
			if string.len(self:GetText()) > 0 and self.enabled then
				for i = 1, size, 1 do
					listPane.lpItem[i]:SetBackgroundColor(1, 1, 1, 0)
					listPane.lpItem[i].selected = false
				end
				self:SetBackgroundColor(1, 1, 1, 0.5)
				self.selected = true
				if listPane.click ~= nil then listPane.click("LEFT") end
			end
		end
		
		listPane.lpItem[i] = item
	end

	function listPane.full()
		return not(listPane.itemCount < listPane.itemMax)
	end

	function listPane.clearSelected()
		for i = 1, listPane.itemMax, 1 do
			listPane.lpItem[i]:SetBackgroundColor(1, 1, 1, 0)
			listPane.lpItem[i].selected = false
		end
	end

	function listPane.itemsEnabled(value)
		for i = 1, listPane.itemMax, 1 do
			listPane.lpItem[i].enabled = value
			if value then
				listPane.lpItem[i]:SetFontColor(1, 1, 1, 1)
			else
				listPane.lpItem[i]:SetFontColor(.5, .5, .5, .5)
			end
		end
	end

	function listPane.itemEnabled(item,value)
		for i = 1, listPane.itemMax, 1 do
			if listPane.lpItem[i]:GetText() == item then
				listPane.lpItem[i].enabled = value
				if value then
					listPane.lpItem[i]:SetFontColor(1, 1, 1, 1)
				else
					listPane.lpItem[i]:SetFontColor(.5, .5, .5, .5)
				end
			end
		end
	end
	
	function listPane.getSelected()
		for i = 1, listPane.itemMax, 1 do
			if listPane.lpItem[i].selected then return listPane.lpItem[i]:GetText() end
		end
		return ""
	end

	function listPane.clearList()
		for i = 1, listPane.itemMax, 1 do
			listPane.lpItem[i]:SetText("")
			listPane.lpItem[i]:SetBackgroundColor(1, 1, 1, 0)
			listPane.lpItem[i].selected = false
		end
	end
	
	function listPane.fillList(items)
		local selectedItem = listPane.getSelected()
		listPane.clearList()
		listPane.itemCount = 0
		for id, details in kUtils.pairsByKeys(items) do
			if not listPane.full() then	
				listPane.itemCount = listPane.itemCount + 1
				listPane.lpItem[listPane.itemCount]:SetText(id)
			end
		end
	end
	
	function listPane.fillTextList(items)
		local selectedItem = listPane.getSelected()
		listPane.clearList()
		listPane.itemCount = 0
		for id, details in pairs(items) do
			if not listPane.full() then	
				listPane.itemCount = listPane.itemCount + 1
				if details == "--" then
					listPane.lpItem[listPane.itemCount]:SetText("")
				else
					listPane.lpItem[listPane.itemCount]:SetText(details)
				end
			end
		end
	end
	
	function listPane.fillListSubSet(items,set)
		local selectedItem = listPane.getSelected()
		listPane.clearList()
		listPane.itemCount = 0
		for id, details in kUtils.pairsByKeys(items) do
			if not listPane.full() and set == string.sub(id,1,string.len(set)) then	
				listPane.itemCount = listPane.itemCount + 1
				listPane.lpItem[listPane.itemCount]:SetText(id)
			end
		end
		return listPane.itemCount
	end

	return listPane
	
end

function kUtils.buildMenuBar(name, parent, height, width, menus)
	local menuBar = UI.CreateFrame("Frame", name, parent)
	menuBar:SetPoint("TOPLEFT", parent, "TOPLEFT", 30, 65)
	menuBar:SetHeight(height)
	menuBar:SetWidth(width)
	menuBar:SetBackgroundColor(0, 0, 0, 0)
	menuBar:SetLayer(0)
	menuBar.menus = {}
	menuBar.outline = kUtils.setOutline(menuBar)
	menuBar.menuIn = false
	
	for id, details in pairs (menus) do
		menu = kUtils.buildLabel(name .. "Menu" .. tostring(id),menuBar,height-7,width,details)
		menu:ClearWidth()
		menu:SetWidth(menu:GetWidth() + 10)
		menu.items = nil
		if id == 1 then
			menu:SetPoint("TOPLEFT", menuBar, "TOPLEFT")
		else
			menu:SetPoint("TOPLEFT", menuBar.menus[menus[id-1]], "TOPRIGHT")
		end
		function menu.Event:LeftClick()
			if string.len(self:GetText()) > 0 then
				if menuBar.click ~= nil then menuBar.click(self:GetText()) end
			end
			if self.items ~= nil then
				local isVisible = not self.items:GetVisible()
				for id, details in pairs (menuBar.menus) do
					if details.items ~= nil then details.items:SetVisible(false) end
				end
				self.items.clearSelected()
				self.items:SetVisible(isVisible)
			end
		end
		menuBar.menus[details] = menu
	end
	
	function menuBar.setItems(menu,width,items)
		local itemList = kUtils.buildListPane(menuBar.menus[menu]:GetName() .. "Items", menuBar.menus[menu], menuBar.menus[menu]:GetHeight() * table.getn(items), width, table.getn(items))
		itemList:SetPoint ("TOPLEFT", menuBar.menus[menu], "BOTTOMLEFT")
		itemList:SetBackgroundColor(.25,.25,.25,1)
		itemList.fillTextList(items)
		itemList:SetVisible(false)
		function itemList.click()
			menuBar.clear()
			if menuBar.click ~= nil then menuBar.click(itemList.getSelected()) end
		end
		menuBar.menus[menu].items = itemList
	end

	function menuBar.itemsEnabled(menu,value)
		menuBar.menus[menu].items.itemsEnabled(value)
	end

	function menuBar.itemEnabled(menu,item,value)
		menuBar.menus[menu].items.itemEnabled(item,value)
	end
	
	function menuBar.Event:MouseOut()
		menuBar.clear()
	end

	function menuBar.clear()
		for id, details in pairs (menuBar.menus) do
			if details.items ~= nil then details.items:SetVisible(false) end
		end
	end
	
	return menuBar
end

function kUtils.buildWindowMover(name, parent)

	local windowMover = UI.CreateFrame("Frame", name, parent)
	windowMover:SetPoint("TOPLEFT", parent, "TOPLEFT")
	windowMover:SetHeight (50)
	windowMover:SetWidth (parent:GetWidth()-40)
	windowMover:SetBackgroundColor(0, 0, 0, 0)

	function windowMover.Event:LeftDown()
		self.MouseDown = true
		mouseData = Inspect.Mouse()
		self.MyStartX = parent:GetLeft()
		self.MyStartY = parent:GetTop()
		self.StartX = mouseData.x - self.MyStartX
		self.StartY = mouseData.y - self.MyStartY
	end
	
	function windowMover.Event:MouseMove(mouseX, mouseY)
		if self.MouseDown then
			parent:SetPoint("TOPLEFT", UIParent, "TOPLEFT", (mouseX - self.StartX), (mouseY - self.StartY))
		end
	end
	
	function windowMover.Event:LeftUp()
		self.MouseDown = false
	end

	return windowMover
	
end

function kUtils.buildScroller(name, parent, height, width, mover)

	local scroller = kUtils.buildFrame(name, parent, height, 12)
	scroller:SetBackgroundColor(0,0,0,0.5)
	
	scroller.Handle = kUtils.buildFrame(name .. "Handle", scroller, height, 12)
	scroller.Handle:SetBackgroundColor(.52,.51,.42)
	scroller.Handle:SetLayer(2)
	scroller.Handle:SetVisible(false)
	
	scroller.Handle.MouseDown = false
	scroller.Handle.StartY = 0
	scroller.Handle.Position = 0
	
	function scroller.Handle.Event:LeftDown()
		self.StartY = Inspect.Mouse().y
		self.Position = self.Controller.Position
		self.MouseDown = true
	end
	function scroller.Handle.Event:MouseMove(x,y)
		if self.MouseDown then
			local offsetY = y - self.StartY
			self.Controller.Position = self.Position + offsetY
			self.Controller:UpdateHandle()
		end
	end
	function scroller.Handle.Event:LeftUp()
		self.MouseDown = false
	end
	function scroller.Handle.Event:LeftUpoutside()
		scroller.Handle.Event.LeftUp(self)
	end
	
	function scroller:UpdateHandle()
		if self.Position < 0 then
			self.Position = 0
		end
		if self.Position > self.Size then
			self.Position = self.Size
		end
		self.Handle:SetPoint("TOPCENTER", self, "TOPCENTER", 0, self.Position)
		if self.Callback then
			self.Callback(math.ceil(self.Position/self.scaler))
		end
	end
	function scroller:SetRange(size, total)
		self.scaler = math.min(size/total,1)
		self.Position = 0
		if self.scaler < 1 then
			self.Handle:SetVisible(true)
			self.Handle:SetHeight(math.ceil(self:GetHeight() * self.scaler))
			self.Size = self:GetHeight() - self.Handle:GetHeight()
			self:UpdateHandle()
		else
			self.Handle:SetVisible(false)
		end
	end
	function scroller:SetPosition(Value)
		if self.Handle:GetVisible() then
			self.Position = self.scaler * Value
			self:UpdateHandle()
		end
	end
	
	scroller.Handle.Controller = scroller

	return scroller

end

function kUtils.makeMovable(object) -- Note: only supports TOPLEFT at this point

	object.movable = true
	
	function object.Event:LeftDown()
		self.MouseDown = self.movable
		mouseData = Inspect.Mouse()
		self.MyStartX = self:GetLeft()
		self.MyStartY = self:GetTop()
		self.StartX = mouseData.x - self.MyStartX
		self.StartY = mouseData.y - self.MyStartY
	end
	
	function object.Event:MouseMove(mouseX, mouseY)
		if self.MouseDown then
			self:SetPoint("TOPLEFT", UIParent, "TOPLEFT", (mouseX - self.StartX), (mouseY - self.StartY))
		end
	end
	
	function object.Event:LeftUp()
		self.MouseDown = false
	end
end

function kUtils.setOutline(item)
	outline = UI.CreateFrame("Frame", item:GetName() .. "Outline", item)
	outline:SetPoint("CENTER", item, "CENTER")
	outline:SetHeight (item:GetHeight())
	outline:SetWidth (item:GetWidth())
	outline:SetBackgroundColor(0,0,0,0)
	outline:SetLayer(item:GetLayer()+1)
	outline.left = UI.CreateFrame("Frame", outline:GetName() .. "Left", outline)
	outline.left:SetPoint("CENTERLEFT", outline, "CENTERLEFT")
	outline.left:SetHeight (outline:GetHeight())
	outline.left:SetWidth (1)
	outline.left:SetBackgroundColor(.45,.44,.38)
	outline.left:SetLayer(outline:GetLayer()+1)
	outline.right = UI.CreateFrame("Frame", outline:GetName() .. "Right", outline)
	outline.right:SetPoint("CENTERRIGHT", outline, "CENTERRIGHT")
	outline.right:SetHeight (outline:GetHeight())
	outline.right:SetWidth (1)
	outline.right:SetBackgroundColor(.52,.51,.42)
	outline.right:SetLayer(outline:GetLayer()+1)
	outline.top = UI.CreateFrame("Frame", outline:GetName() .. "Top", outline)
	outline.top:SetPoint("CENTERTOP", outline, "CENTERTOP")
	outline.top:SetHeight (1)
	outline.top:SetWidth (outline:GetWidth())
	outline.top:SetBackgroundColor(.52,.51,.42)
	outline.top:SetLayer(outline:GetLayer()+1)
	outline.bottom = UI.CreateFrame("Frame", outline:GetName() .. "Bottom", outline)
	outline.bottom:SetPoint("CENTERBOTTOM", outline, "CENTERBOTTOM")
	outline.bottom:SetHeight (1)
	outline.bottom:SetWidth (outline:GetWidth())
	outline.bottom:SetBackgroundColor(.45,.44,.38)
	outline.bottom:SetLayer(outline:GetLayer()+1)
	return outline
end
