local info, data = ...
local me = {Name = "myUI Config", Addon = info.identifier, SettingsKey = "Config", ModuleVersion = "0.2.2", ReleaseType = "(public beta)"};
table.insert(Event.Addon.Load.End, {function(addon) if (addon == me.Addon) then myUI:RegisterModule(me)  end end, me.Addon, "_startup"})

myUI.Configuration = me

local L
local isnil, ifnil, notnil, iif, minmax
local __showConfig

function me:HandleSlashCommand(commandline)
	if (commandline == "config") then
		__showConfig(self)
		return true
	end
end

function me:GetSlashCommandDescriptions()
	local r = {}
	table.insert(r, "config - " .. L["Open Config"])
	return r
end


function me:GetDefaults()
	local defaults = {
		FirstRun = true,
		Enabled = true,
		Version = self.ModuleVersion				
	}
	return defaults
end

function me:Initialize(profile)
	isnil = myUI.global.isnil
	ifnil = myUI.global.ifnil
	notnil = myUI.global.notnil
	iif = myUI.global.iif
	minmax = myUI.global.minmax
	
	L = self:LoadLanguage()
	
	if (not myUI_Account.ConfigUI) then myUI_Acount.ConfigUI = {} end
	
	
end

__showConfig = function(self)
	local f = self._configForm
	
	if (not f) then
		local cx = 940
		local cy = 680
		
		local ifnil = myUI.global.ifnil
		local iif = myUI.global.iif
		
		local white = myUI.colors.White
		local black = myUI.colors.Black
		local transparent = myUI.color:new(0,0,0,0)
		local activebg = myUI.color:clone(black, 100)
		local highlight = myUI.colors.DodgerBlue
		local active = myUI.colors.DeepSkyBlue
		
		f = myUI:CreateFrame("RiftWindow")
		f:SetLayer(1000)
		f:SetController("content")
		f:SetVisible(false)		
		f:SetWidth(cx)
		f:SetHeight(cy)
		f:SetTitle("myUI " .. L["Configuration"])
		
		local x = ifnil(myUI_Account.ConfigUI.x, (UIParent:GetWidth() - cx) / 2)
		local y = ifnil(myUI_Account.ConfigUI.y, (UIParent:GetHeight() - cy) / 2)
		local borderX = 18
		local borderY = 12
		local left, top, right, bottom = f:GetTrimDimensions()
		
		f:SetPoint("TOPLEFT", UIParent, "TOPLEFT", x, y)
		
		f.Header = myUI:CreateFrame("Text", f)
		if (f.Header) then
			f.Header:SetPoint("BOTTOMLEFT", f, "BOTTOMLEFT", borderX, -borderY)
			
			f.Header:SetFontSize(14)
			f.Header:SetHeight(20)
			f.Header:SetFontColor(white:rgba())
			f.Header:SetWordwrap(false)
			f.Header:SetText("myUI Configuration - Version " .. self.ModuleVersion)
		end
		
		f.CloseButton = myUI:CreateFrame("RiftButton", f)
		if (f.CloseButton) then
			f.CloseButton:SetText(L["Close"])
			f.CloseButton:SetPoint("BOTTOMRIGHT", f, "BOTTOMRIGHT", -borderX, -4)			
			f.Header:SetPoint("BOTTOMRIGHT", f.CloseButton, "BOTTOMLEFT", - borderX, -4)
		
			f.CloseButton.Event.LeftPress = function(btn)
				local p = btn:GetParent()
				
				-- TODO: update settings, etc, etc
				
				for _, mbtn in ipairs(btn:GetParent().ModuleButtons) do
					if (mbtn.Module) and (mbtn.Module.HideMovers) then
						mbtn.Module:HideMovers()
					end
				end
				
				myUI_Account.ConfigUI.x = p:GetLeft()
				myUI_Account.ConfigUI.y = p:GetTop()
			
				p:SetVisible(false)
			end
			
			f._setVisible = f.SetVisible
			f.SetVisible = function(self, visible)
				if (visible) then
					if (self.ModuleButtons) then
						for _, mbtn in ipairs(self.ModuleButtons) do							
							if (mbtn.isSelected) and (mbtn.Module) then
								if (mbtn.ShowMovers) and (mbtn.Module ~= myUI) then
									mbtn.Module:ShowMovers()
								end
								if (mbtn.Module.OnConfigActive) then
									mbtn.Module:OnConfigActive()
								end
							end
						end
					end
				else
					for _, mbtn in ipairs(self.ModuleButtons) do
						if (mbtn.isSelected) and (mbtn.Module) then
							if (mbtn.Module.OnConfigInactive) then
								mbtn.Module:OnConfigInactive()
							end
						end
					end
				end
				self:_setVisible(visible)
			end
		end
		
		
		f.Dragger = myUI.Widgets.Dragbox:new(f)
		if (f.Dragger) then		
			f.Dragger:SetHeight(top/2)		
			f.Dragger:SetPoint("TOPLEFT", f, "TOPLEFT", -left, -top * 0.75)
			f.Dragger:SetPoint("TOPRIGHT", f, "TOPRIGHT", right, -top* 0.75)
		end
				
		f.ModuleBox = myUI.Widgets.BorderBox:new(f)
		if (f.ModuleBox) then
			f.ModuleBox:SetPoint("TOPLEFT", f, "TOPLEFT", borderX, borderY)
			f.ModuleBox:SetPoint("BOTTOMLEFT", f, "BOTTOMLEFT", borderX, - borderY - 32)
			f.ModuleBox:SetWidth(200)			
		end
		
		f.ConfigBox = myUI.Widgets.BorderBox:new(f)
		f.ConfigBox:SetLayer(f:GetLayer() +1)
		if (f.ConfigBox) then
			f.ConfigBox:SetPoint("TOPLEFT", f.ModuleBox, "TOPRIGHT", borderX, 0)
			--f.ConfigBox:SetPoint("BOTTOMLEFT", f.ModuleBox, "BOTTOMRIGHT", borderX, 0)
			--f.ConfigBox:SetPoint("CENTERRIGHT", f, "CENTERRIGHT", -borderX, 0)
			
			f.ConfigBox:SetPoint("BOTTOMRIGHT", f, "BOTTOMRIGHT", -borderX, -borderY - 32)
		end
		
		local mlist = {}
		for _, mod in pairs(myUI.Modules) do
			table.insert(mlist, mod)
		end
		
		f.ModuleButtons = {}
		
		local function __selectMBTN(mbtn, selected)
			mbtn.isSelected = selected
			if (selected) then 
				mbtn:SetFontColor(active:rgba())
				mbtn:SetBackgroundColor(activebg:rgba())
				
				if (not mbtn.ConfigFrame) then
					local cframe
					local p = mbtn:GetParent().ConfigBox
					if (mbtn.Module.GetConfigFrame) then
						cframe = mbtn.Module:GetConfigFrame(p)
					end
					
					if (not cframe) then
						cframe = myUI:CreateFrame("Frame", p)
						cframe:SetVisible(false)
						cframe.Text = myUI:CreateFrame("Text", cframe)						
						cframe.Text:SetText(L["No Config"])
						cframe.Text:SetPoint("TOPLEFT", cframe, "TOPLEFT", 16, 16)
						cframe.Text:SetPoint("TOPRIGHT", cframe, "TOPRIGHT", -16, 16)
						cframe.Text:SetHeight(20)
					end
					
					cframe:SetPoint("TOPLEFT", p, "TOPLEFT", 0, 0)
					cframe:SetPoint("BOTTOMRIGHT", p, "BOTTOMRIGHT", 0 , 0)	
					
					mbtn.ConfigFrame = cframe
					
				end
				
				if (mbtn.Module ~= myUI) and (mbtn.Module.ShowMovers) then					
					mbtn.Module:ShowMovers()
				end			
				if (mbtn.ConfigFrame) then 
					mbtn.ConfigFrame:SetVisible(true) 					
				end
				if (mbtn.Module) and (mbtn.Module.OnConfigActive) then
					mbtn.Module:OnConfigActive()
				end
			else
				if (mbtn.ConfigFrame) then 
					mbtn.ConfigFrame:SetVisible(false) 										
				end
				mbtn:SetFontColor(white:rgba())
				mbtn:SetBackgroundColor(transparent:rgba())								
	
				if (mbtn.Module) and (mbtn.Module.OnConfigInactive) then
					mbtn.Module:OnConfigInactive()
				end
	
				if (mbtn.Module ~= myUI) and (mbtn.Module.HideMovers) then
					mbtn.Module:HideMovers()
				end
			end
			
		end
		
		local function __mouseInMBTN(mbtn)
			if (not mbtn.isSelected) then
				mbtn:SetFontColor(highlight:rgba())
			end
		end
		
		local function __mouseOutMBTN(mbtn)
			if (not mbtn.isSelected) then
				mbtn:SetFontColor(white:rgba())
			end
		end

		local function __leftClickMBTN(mbtn)
			if (not mbtn.isSelected) then			
				for _, b in ipairs(mbtn:GetParent().ModuleButtons) do
					
				
					b:SetSelected(false)
				end
				
				
				mbtn:SetSelected(true)				
			end
		end
		
		
		
		
		local myuibtn = myUI:CreateFrame("Text", f)		
		myuibtn.isSelected = true
		
		myuibtn.Module = myUI					
		myuibtn:SetWordwrap(true)
		
		myuibtn:SetPoint("TOPLEFT", f.ModuleBox, "TOPLEFT", 4, 4)
		myuibtn:SetPoint("TOPRIGHT", f.ModuleBox, "TOPRIGHT", -4, 4)
		myuibtn:SetText("myUI - v" .. myUI.InternalVersion)
		myuibtn:SetHeight(myuibtn:GetFullHeight())
		
		myuibtn.Event.MouseIn = __mouseInMBTN
		myuibtn.Event.MouseOut = __mouseOutMBTN
		myuibtn.Event.LeftClick = __leftClickMBTN
		myuibtn.SetSelected = __selectMBTN
		
		table.insert(f.ModuleButtons, myuibtn)
		
		local ym = 0
		local xm = 0
		local anchorm = myuibtn
		
		for _, mod in ipairs(mlist) do
			local mbtn = myUI:CreateFrame("Text", f)
			
			mbtn.Module = mod		
			mbtn:SetWordwrap(true)
			
			mbtn:SetPoint("TOPLEFT", anchorm, "BOTTOMLEFT", xm, ym)
			mbtn:SetPoint("TOPRIGHT", anchorm, "BOTTOMRIGHT", -xm, ym)
			mbtn:SetText(mod.Name .. " - v" .. mod.ModuleVersion)
			
			local cym = mbtn:GetFullHeight() + 4
			mbtn:SetHeight(cym)
			
			mbtn.Event.MouseIn = __mouseInMBTN
			mbtn.Event.MouseOut = __mouseOutMBTN
			mbtn.Event.LeftClick = __leftClickMBTN
			mbtn.SetSelected = __selectMBTN
			
			table.insert(f.ModuleButtons, mbtn)
			anchorm = mbtn
			
		end
		
		myuibtn:SetSelected(true)
		
		
		self._configForm = f
	end
	
	f:SetVisible(true)
	
end
