local ifnil = myUI.global.ifnil
local isnil = myUI.global.isnil
local notnil = myUI.global.notnil
local minmax = myUI.global.minmax
local inherit = myUI.global.inherit
local tohex = myUI.string.tohex

local function __rgba(self) 
	return
		minmax(self._r/255, 0, 1),
		minmax(self._g/255, 0, 1),
		minmax(self._b/255, 0, 1),
		minmax(self._a/255, 0, 1)    
end

local function __tostring(self)
	if isnil(self._name) then
		return string.format("Color{#%s%s%s%s}",
			tohex(self._r), tohex(self._g), tohex(self._b), tohex(self._a)
			)
	else
		return string.format("Color{%s}", self._name)
	end
end

local function __fade(self, value)
	if (value) and (type(value) == "number") then
		self._a = minmax(self._a - value, 0, 255)
	end
	
	return self
end

local function __darken(self, value)
	if (value) and (type(value) == "number") then
		self._r = minmax(self._r - value, 0, 255)
		self._g = minmax(self._g - value, 0, 255)
		self._b = minmax(self._b - value, 0, 255)
		self._name = nil
	end
	
	return self
end

local function __lighten(self, value)
	if (value) and (type(value) == "number") then
		self._r = minmax(self._r + value, 0, 255)
		self._g = minmax(self._g + value, 0, 255)
		self._b = minmax(self._b + value, 0, 255)
		self._name = nil
	end
	
	return self
end

local function __getName(self) return self._name; end

local color = {
	new = function(self, r, g, b, a, name)
		local this = {_r = r, _g = g, _b = b, _a = ifnil(a, 255), _name = name}

		this.getName = __getName
		this.__tostring = __tostring
		this.rgba = __rgba		
		this.fade = __fade		
		this.darken = __darken
		this.lighten = __lighten
		
		return this 
	end,
	
	clone = function(self, baseClr, a)
		return self:new(baseClr._r, baseClr._g, baseClr._b, ifnil(a, baseClr._a))
	end,	
	
	fromRgba = function(self, r, g, b, a)
		return self:new(minmax(r*255,0,255), minmax(g*255,0,255), minmax(b*255,0,255))
	end
}

myUI.color = color;

local colors = {
    AliceBlue         = color:new(240,248,255,nil,"Alice Blue"),
    AntiqueWhite      = color:new(250,235,215,nil,"Antique White"),
    Aqua              = color:new(0,255,255,nil,"Aqua"),
    Aquamarine        = color:new(127,255,212,nil,"Aquamarine"),
    Azure             = color:new(240,255,255,nil,"Azure"),
    Beige             = color:new(245,245,220,nil,"Beige"),
    Bisque            = color:new(255,228,196,nil,"Bisque"),
    Black             = color:new(0,0,0,nil,"Black"),
    BlanchedAlmond    = color:new(255,235,205,nil,"Blanched Almond"),
    Blue              = color:new(0,0,255,nil,"Blue"),
    BlueViolet        = color:new(138,43,226,nil,"Blue-Violet"),
    Brown             = color:new(165,42,42,nil,"Brown"),
    Burlywood         = color:new(222,184,135,nil,"Burlywood"),
    CadetBlue         = color:new(95,158,160,nil,"Cadet Blue"),
    Chartreuse        = color:new(127,255,0,nil,"Chartreuse"),
    Chocolate         = color:new(210,105,30,nil,"Chocolate"),
    Coral             = color:new(255,127,80,nil,"Coral"),
    CornflowerBlue    = color:new(100,149,237,nil,"Cornflower Blue"),
    Cornsilk          = color:new(255,248,220,nil,"Cornsilk"),
    Cyan              = color:new(0,255,255,nil,"Cyan"),
    DarkBlue          = color:new(0,0,139,nil,"Dark Blue"),
    DarkCyan          = color:new(0,139,139,nil,"Dark Cyan"),
    DarkGoldenrod     = color:new(184,134,11,nil,"Dark Goldenrod"),
    DarkGray          = color:new(169,169,169,nil,"Dark Gray"),
    DarkGreen         = color:new(0,100,0,nil,"Dark Green"),
    DarkKhaki         = color:new(189,183,107,nil,"Dark Khaki"),
    DarkMagenta       = color:new(139,0,139,nil,"Dark Magenta"),
    DarkOliveGreen    = color:new(85,107,47,nil,"Dark Olive-Green"),
    DarkOrange        = color:new(255,140,0,nil,"Dark Orange"),
    DarkOrchid        = color:new(153,50,204,nil,"Dark Orchid"),
    DarkRed           = color:new(139,0,0,nil,"Dark Red"),
    DarkSalmon        = color:new(233,150,122,nil,"Dark Salmon"),
    DarkSeaGreen      = color:new(143,188,143,nil,"Dark Sea Green"),
    DarkSlateBlue     = color:new(72,61,139,nil,"Dark Slate Blue"),
    DarkSlateGray     = color:new(47,79,79,nil,"Dark Slate Gray"),
    DarkTurquoise     = color:new(0,206,209,nil,"Dark Turquoise"),
    DarkViolet        = color:new(148,0,211,nil,"Dark Violet"),
    DeepPink          = color:new(255,20,147,nil,"Deep Pink"),
    DeepSkyBlue       = color:new(0,191,255,nil,"Deep Sky Blue"),
    DimGray           = color:new(105,105,105,nil,"Dim Gray"),
    DodgerBlue        = color:new(30,144,255,nil,"Dodger Blue"),
    Firebrick         = color:new(178,34,34,nil,"Firebrick"),
    FloralWhite       = color:new(255,250,240,nil,"Floral White"),
    ForestGreen       = color:new(34,139,34,nil,"Forest Green"),
    Fuschia           = color:new(255,0,255,nil,"Fuschia"),
    Gainsboro         = color:new(220,220,220,nil,"Gainsboro"),
    GhostWhite        = color:new(255,250,250,nil,"Ghost White"),
    Gold              = color:new(255,215,0,nil,"Gold"),
    Goldenrod         = color:new(218,165,32,nil,"Goldenrod"),
    Gray              = color:new(128,128,128,nil,"Gray"),
    Green             = color:new(0,128,0,nil,"Green"),
    GreenYellow       = color:new(173,255,47,nil,"Green-Yellow"),
    Honeydew          = color:new(240,255,240,nil,"Honeydew"),
    HotPink           = color:new(255,105,180, nil, "Hot Pink"),
    IndianRed         = color:new(205,92,92, nil, "Indian Red"),
    Ivory             = color:new(255,255,240, nil, "Ivory"),
    Khaki             = color:new(240,230,140,nil,"Khaki"),
    Lavender          = color:new(230,230,250,nil,"Lavender"),
    LavenderBlush     = color:new(255,240,245,nil,"Lavender-Blush"),
    LawnGreen         = color:new(124,252,0,nil,"Lawn Green"),
    LemonChiffon      = color:new(255,250,205,nil,"Lemon-Chiffon"),
    LightBlue         = color:new(173,216,230,nil,"Light Blue"),
    LightCoral        = color:new(240,128,128,nil,"Light Coral"),
    LightCyan         = color:new(244,255,255,nil,"Light Cyan"),
    LightGoldenrod    = color:new(238,221,130,nil,"Light Goldenrod"),
    LightGoldenrodYellow = color:new(250,250,210,nil,"Light Goldenrod-Yellow"),
    LightGray         = color:new(211,211,211,nil,"Light Gray"),
    LightGreen        = color:new(144,238,144,nil,"Light Green"),
    LightPink         = color:new(255,182,193,nil,"Light Pink"),
    LightSalmon       = color:new(255,160,122, nil,"Light Salmon"),
    LightSeaGreen     = color:new(32,178,170,nil,"Light Sea Green"),
    LightSkyBlue      = color:new(135,206,250,nil,"Light Sky Blue"),
    LightSlateBlue    = color:new(132,112,255,nil,"Light Slate Blue"),
    LightSlateGray    = color:new(119,136,153,nil,"Light Slate Gray"),
    LightSteelBlue    = color:new(176,196,222,nil,"Light Steel Blue"),
    LightYellow       = color:new(255,255,224, nil,"Light Yellow"),
    Lime              = color:new(0,255,0,nil,"Lime"),
    LimeGreen         = color:new(50,205,50,nil,"Lime-Green"),
    Linen             = color:new(250,240,230,nil,"Linen"),
    Magenta           = color:new(255,0,255,nil,"Magenta"),
    Maroon            = color:new(128,0,0,nil,"Maroon"),
    MediumAquamarine  = color:new(102,205,170,nil,"Medium Aquamarine"),
    MediumBlue        = color:new(0,0,205,nil,"Medium Blue"),
    MediumOrchid      = color:new(186,85,211,nil,"Medium Orchid"),
    MediumPurple      = color:new(147,112,219,nil,"Medium Purple"),
    MediumSeaGreen    = color:new(60,179,113,nil,"Medium Sea Green"),
    MediumSlateBlue   = color:new(123,104,238,nil,"Medium Slate Blue"),
    MediumSpringGreen = color:new(0,250,154,nil,"Medium Spring Green"),
    MediumTurquoise   = color:new(72,209,204,nil,"Medium Turquoise"),
    MediumVioletRed   = color:new(199,21,133, nil, "Medium Violet-Red"),
    MidnightBlue      = color:new(25,25,112,nil,"Midnight Blue"),
    MintCream         = color:new(245,255,250,nil,"Mint Cream"),
    MistyRose         = color:new(255,228,225,nil,"Misty Rose"),
    Moccasin          = color:new(255,228,181,nil,"Moccasin"),
    NavajoWhite       = color:new(255,222,173,nil,"Navajo White"),
    Navy              = color:new(0,0,128,nil,"Navy"),
    OldLace           = color:new(253,245,230,nil,"Old Lace"),
    Olive             = color:new(128,128,0,nil,"Olive"),
    OliveDrab         = color:new(107,142,35,nil,"Olive-Drab"),
    Orange            = color:new(255,165,0,nil,"Orange"),
    OrangeRed         = color:new(255,69,0,nil,"Orange-Red"),
    Orchid            = color:new(218,112,214,nil,"Orchid"),
    PaleGoldenrod     = color:new(238,232,170,nil,"Pale Goldenrod"),
    PaleGreen         = color:new(152,251,152,nil,"Pale Green"),
    PaleTurquoise     = color:new(175,238,238, nil, "Pale Turquoise"),
    PaleVioletRed     = color:new(219,112,147,nil,"Pale Violet-Red"),
    PapayaWhip        = color:new(255,239,213,nil,"Papaya Whip"),
    PeachPuff         = color:new(255,218,185,nil,"Peach Puff"),
    Peru              = color:new(205,113,63,nil,"Peru"),
    Pink              = color:new(255,192,203,nil,"Pink"),
    Plum              = color:new(221,160,221,nil,"Plum"),
    PowderBlue        = color:new(176,224,230,nil,"Powder Blue"),
    Purple            = color:new(128,0,128,nil,"Purple"),
    Red               = color:new(255,0,0,nil,"Red"),
    RosyBrown         = color:new(188,143,143,nil,"Rosy Brown"),
    RoyalBlue         = color:new(65,105,225,nil,"Royal Blue"),
    SaddleBrown       = color:new(139,69,19,nil,"Saddle Brown"),
    Salmon            = color:new(250,128,114,nil,"Salmon"),
    SandyBrown        = color:new(244,164,96,nil,"Sandy Brown"),
    SeaGreen          = color:new(46,139,87,nil,"Sea Green"),
    Seashell          = color:new(255,245,238,nil,"Sea Shell"),
    Sienna            = color:new(160,82,45,nil,"Sienna"),
    Silver            = color:new(192,192,192,nil,"Silver"),
    SkyBlue           = color:new(135,206,235,nil,"Sky Blue"),
    SlateBlue         = color:new(106,90,205,nil,"Slate Blue"),
    SlateGray         = color:new(112,128,144,nil,"Slate Gray"),
    Snow              = color:new(255,250,250,nil,"Snow"),
    SpringGreen       = color:new(0,255,127,nil,"Spring Green"),
    SteelBlue         = color:new(70,130,180,nil,"Steel Blue"),
    Tan               = color:new(210,180,140, nil, "Tan"),
    Teal              = color:new(0,128,128,nil,"Teal"),
    Thistle           = color:new(216,191,216,nil,"Thistle"),
    Tomato            = color:new(255,99,71,nil,"Tomato"),
    Turquoise         = color:new(64,224,208,nil,"Turquoise"),
    Violet            = color:new(238,130,238,nil,"Violet"),
    VioletRed         = color:new(208,32,144,nil,"Violet-Red"),
    Wheat             = color:new(245,222,179,nil,"Wheat"),
    White             = color:new(255,255,255,nil,"White"),
    WhiteSmoke        = color:new(245,245,245,nil,"White Smoke"),
    Yellow            = color:new(255,255,0,nil,"Yellow"),
    YellowGreen       = color:new(154,205,50, nil,"Yellow-Green"),
    Transparent       = color:new(0,0,0,0,"Transparent")     

}
myUI.colors = colors