
	
function myUI:GetAccountDefaults()
	local defaults = {
		Version = myUI.InternalVersion,
		UpdateSpeed = 20,
		UnitUpdateSpeed = 6,
		AdaptiveUpdate = true,
		Profiles = {
			["default"] = {
				globalFont = {src = "myUI", name = "Arial", file = "fonts/arial.ttf", defaultSize = 15}
			}
		
		},
		Fonts = {
			{src = "Rift", name = "Rift Default", file = "$Flareserif_medium", defaultSize = 14},
			{src = "myUI", name = "Arial", file = "fonts/arial.ttf", defaultSize = 15},
			{src = "myUI", name = "Excite", file = "fonts/EXCITE__.ttf"},
			{src = "myUI", name = "Groovy Fast", file = "fonts/Groovy Fast.ttf"},
			{src = "myUI", name = "Highland Gothic", file = "fonts/HighlandGothicFLF.ttf", defaultSize = 12},
			{src = "myUI", name = "Liberation", file = "fonts/LiberationMono-Regular.ttf"},
			{src = "myUI", name = "Permanent Marker", file = "fonts/PermanentMarker.ttf"},
			{src = "myUI", name = "Sansumi", file = "fonts/Sansumi-Bold.ttf"},
			
		
		},
		ConfigUI = {},		-- x, y
		Lists = {}, -- [name] = {["item name"] = true }
	}
	
	return defaults
end

function myUI:CheckAccountDefaults(settings)
	-- This function checks to see if our settings have been created/saved and also
	-- checks to make sure they contain each section our default settings have, in case
	-- we've added a new section.
	
	local defaults = myUI:GetAccountDefaults()
	
	if (settings == nil) then
		settings = defaults
	else
		-- TODO: version check when we need it for something
		settings.Version = myUI.InternalVersion
		
		
		if (settings.Profiles == nil) then settings.Profiles = defaults.Profiles end		
		if (settings.Fonts == nil) then settings.Fonts = defaults.Fonts end		
		if (settings.UpdateSpeed == nil) then settings.UpdateSpeed = defaults.UpdateSpeed end
		if (settings.UnitUpdateSpeed == nil) then settings.UnitUpdateSpeed = defaults.UnitUpdateSpeed end
		if (settings.AdaptiveUpdate == nil) then settings.AdaptiveUpdate = defaults.AdaptiveUpdate end
		if (settings.Lists == nil) then settings.Lists = defaults.Lists end
		local defprofile = defaults.Profiles["default"];
		
		for _, profile in pairs(settings.Profiles) do
			for defsection, defvalues in pairs(defprofile) do
				if (profile[defsection] == nil) then					
					profile[defsection] = defvalues
				end
			end						
		end

		if (settings.ConfigUI == nill) then
			settings.ConfigUI = defaults.ConfigUI
		end
		
	
	end
	
	return settings
end

local _configFrame
local white = myUI.colors.White
local black = myUI.colors.Black
local transparent = myUI.color:new(0,0,0,0)
local activebg = myUI.color:clone(black, 100)	
local active = myUI.colors.DeepSkyBlue

local ifnil = myUI.global.ifnil
local iif = myUI.global.iif
local minmax = myUI.global.minmax

function myUI:CheckCharacterDefaults(settings)
	if (settings == nil) then
		settings = {
			Version = myUI.InternalVersion,
			CurrentProfile = "default"

		}
	else
		settings.Version = myUI.InternalVersion -- update to this version		
	end
	return settings
end


function myUI:GetConfigFrame(parent)
	if (not _configFrame) then
		local L = myUI._L
	
		_configFrame = myUI:CreateFrame("Frame", parent)	
		_configFrame:SetLayer(parent:GetLayer() + 1)
	
		local activebg = myUI.color:clone(black, 100)
	
		local border = 16	
		local tabLabels = {
			L["General"],
			L["Profiles"],
			L["Fonts"],
			L["Lists"],
			L["Performance"],			

		}
		
		local TAB_GENERAL = 1
		local TAB_PROFILES = 2
		local TAB_FONTS = 3
		local TAB_LISTS = 4
		local TAB_PERFORMANCE = 5
		
		
		local f = myUI.Widgets.Tabstrip:new(tabLabels, "VERTICAL" , _configFrame)
		local bgclr = f:GetActiveBgColor()
		
		for i = 1, #tabLabels, 1 do
			local t = f:GetTab(i)
			
			local tpage = myUI:CreateFrame("Frame", _configFrame)
			--tpage:SetPoint("TOPLEFT", f, "TOPRIGHT")
			tpage:SetPoint("TOPLEFT", _configFrame, "TOPLEFT", border + f:GetWidth(), border)
			tpage:SetPoint("BOTTOMRIGHT", _configFrame, "BOTTOMRIGHT", -border, -border)
			tpage:SetBackgroundColor(bgclr:rgba())		
			--tpage:SetBackgroundColor(1,0,0,0.5)
			tpage:SetVisible(i == 1)
			
			t.tabpage = tpage					
		end
	
		f:ClearAll()
		f:SetPoint("TOPLEFT", _configFrame, "TOPLEFT", border, border)
		f:SetPoint("BOTTOMRIGHT", _configFrame, "BOTTOMRIGHT", -border, -border)
		--f:SetPoint("TOPRIGHT", _configFrame, "TOPRIGHT", -border, border)
		f:SetVisible(true)		
		f:SetSelectedIndex(1)
		f.SelectedIndexChanged = function(tabstrip, index)
			
			
			for i = 1, tabstrip:GetTabCount(), 1 do
				tabstrip:GetTab(i).tabpage:SetVisible(i == index)
			end
		end
		
		f:SetSelectedIndex(1)
		_configFrame.tabstrip = f
	
	
		if (true) then -- general tab page
		local stabpage = _configFrame.tabstrip:GetTab(TAB_GENERAL).tabpage
		
		f = myUI:CreateFrame("Text", stabpage)
		f:SetPoint("TOPLEFT", stabpage, "TOPLEFT", border, border)
		--f:SetPoint("TOPRIGHT", stabpage, "TOPRIGHT", -border, border)
		--f:SetPoint("BOTTOMLEFT", stabpage, "BOTTOMLEFT", border, -border)
		f:SetPoint("BOTTOMRIGHT", stabpage, "BOTTOMRIGHT", -border, -border)
		
		f:SetWordwrap(true)			
		local msg = L["GeneralMsg"] .. "\n\n"
		local cmds = myUI:GetUsage()
		if (cmds) then
			for _, ln in ipairs(cmds) do
				msg = msg .. ln .. "\n"
			end
		end
		f:SetText(msg)
		end
		
		if (true) then -- profiles tab page
		stabpage = _configFrame.tabstrip:GetTab(TAB_PROFILES).tabpage
		
		f = myUI:CreateFrame("Text", stabpage)		
		f:SetWordwrap(true)			
		f:SetText(L["ProfilesMsg"])
		f:SetPoint("TOPLEFT", stabpage, "TOPLEFT", border, border)
		f:SetPoint("TOPRIGHT", stabpage, "TOPRIGHT", -border, border)
		--f:SetHeight(f:GetFullHeight())
		stabpage.lblProfileInfo = f
		
		f = myUI:CreateFrame("Text", stabpage)
		f:Autosize(L["Current Profiles"])
		f:SetPoint("TOPLEFT", stabpage, "TOPLEFT", border, 260)
		f:SetPoint("TOPRIGHT", stabpage, "TOPRIGHT", -border, 260)
		stabpage.lblCurrentProfile = f
		
		f = myUI.Widgets.Listbox:new(stabpage)
		f:SetPoint("TOPLEFT", stabpage.lblCurrentProfile, "BOTTOMLEFT", 0, 4)
		f:SetPoint("BOTTOMLEFT", stabpage, "BOTTOMLEFT", border, - border)
		f:SetWidth(200)
		stabpage.lstProfiles = f
		
		for name, _ in pairs(myUI_Account.Profiles) do
			f:AddItem(tostring(name))
		end
		
		f.SelectedIndexChanged = function(lstProfiles, selindex)
			local spage = lstProfiles:GetParent()
			
			spage.btnActivateProfile:SetEnabled(selindex > 0)
			spage.btnCopyProfile:SetEnabled(selindex > 0)
			spage.btnDeleteProfile:SetEnabled(lstProfiles:GetSelectedItem() ~= "default")
		end
		
		f = myUI:CreateFrame("RiftButton", stabpage)
		f:SetText(L["Activate Profile"])
		f:SetPoint("TOPLEFT", stabpage.lstProfiles, "TOPRIGHT", border, 0)
		f:SetEnabled(false)
		f.Event.LeftPress = function(btnActivateProfile)
			local spage = btnActivateProfile:GetParent()
			local pname = spage.lstProfiles:GetSelectedItem()
			if (pname) then
				myUI_Character.CurrentProfile = pname
				myUI.msgbox(spage, pname .. L["Profile Activated"], _, "OK", "INFO")
			end
		end
		stabpage.btnActivateProfile = f
		
		f = myUI:CreateFrame("Text", stabpage)
		f:Autosize(L["Active Profile"] .. myUI_Character.CurrentProfile)
		f:SetPoint("TOPLEFT", stabpage.btnActivateProfile, "BOTTOMLEFT", 0, 0)
		stabpage.lblActiveProfile = f
		
		f = myUI:CreateFrame("RiftButton", stabpage)
		f:SetText(L["New Profile"])
		f:SetPoint("TOPLEFT", stabpage.btnActivateProfile, "BOTTOMLEFT", 0, 36)
		f.Event.LeftPress = function(btnNewProfile)
			local spage = btnNewProfile:GetParent()
			
			myUI.inputbox(spage, L["New Profile Instructions"], _, 
				function(result, text, spage)
					if (result == "OK") then
						text = myUI.string.trim(text)
						if (string.len(text) > 0) then
							if (myUI_Account.Profiles[text]) then
								myUI.msgbox(spage, L["Profile Already Exists"], _, "OK", "INFO")
							else
								myUI_Account.Profiles[text] = {}
								myUI.msgbox(spage, L["Profile Created"], _, "OK", "INFO")
								spage.lstProfiles:AddItem(text)
							end
						end
					end					
				end,
			spage)
		
		
		
		end
		stabpage.btnNewProfile = f
		
		f = myUI:CreateFrame("RiftButton", stabpage)
		f:SetText(L["Copy Profile"])
		f:SetPoint("TOPLEFT", stabpage.btnNewProfile, "BOTTOMLEFT", 0, -6)
		f:SetEnabled(false)
		f.Event.LeftPress = function(btnCopyProfile)
			local spage = btnCopyProfile:GetParent()
			
			myUI.inputbox(spage, L["New Profile Instructions"], _, 
				function(result, text, spage)
					if (result == "OK") then
						text = myUI.string.trim(text)
						local copy = spage.lstProfiles:GetSelectedItem()
						if (string.len(text) > 0) and (copy) then
							if (myUI_Account.Profiles[text]) then
								myUI.msgbox(spage, L["Profile Already Exists"], _, "OK", "INFO")
							else
								myUI_Account.Profiles[text] = myUI.global.shallowCopy(myUI_Account.Profiles[copy])								
								
								
								myUI.msgbox(spage, L["Profile Created"], _, "OK", "INFO")
								spage.lstProfiles:AddItem(text)
							end
						end
					end					
				end,
			spage)
		end
		stabpage.btnCopyProfile = f
		
		f = myUI:CreateFrame("RiftButton", stabpage)
		f:SetText(L["Delete Profile"])
		f:SetPoint("TOPLEFT", stabpage.btnCopyProfile, "BOTTOMLEFT", 0, -6)
		f:SetEnabled(false)
		f.Event.LeftPress =  function(btnDeleteProfile)
			local spage = btnDeleteProfile:GetParent()
			local pname = spage.lstProfiles:GetSelectedItem()
			if ((nil ~= pname) and (pname ~= "default")) then
				myUI.msgbox(spage, L["Confirm Profile Delete"], _, "YESNO", "QUESTION", 
					function(result, state)
						if (result == "YES") then
							myUI_Account.Profiles[state[1]] = nil
							state[2].lstProfiles:RemoveItem(state[2].lstProfiles:GetSelectedIndex())
						end
					end, {pname, spage}
				)
			end
		end
		stabpage.btnDeleteProfile = f
		
		end
		
		-- fonts tab page
		if (true) then
		stabpage = _configFrame.tabstrip:GetTab(TAB_FONTS).tabpage
		
		f  = myUI:CreateFrame("Text", stabpage)
		f:SetPoint("TOPLEFT", stabpage, "TOPLEFT", border, border)
		f:SetPoint("TOPRIGHT", stabpage, "TOPRIGHT", -border, border)
		f:SetHeight(50)
		f:SetWordwrap(true)
		f:SetText(L["FontsMsg"])
		
		local _loading = false
		local function _fontChecked(checkbox)
			if (_loading) then return end
			if (checkbox:GetChecked()) then				
				local tpage = _configFrame.tabstrip:GetTab(TAB_FONTS).tabpage				
				for _, chk in ipairs(tpage.chkFonts) do					
					if (checkbox.index ~= chk.index) then											
						chk:SetChecked(false)
					else
						_loadFont(chk.font)
					end
				end
			end
		end
		
		local function _loadFonts()
			_loading = true
			local defaultFont = ""
			if (myUI.CurrentProfile.globalFont) then
				defaultFont = myUI.CurrentProfile.globalFont.name
			end
			local tpage = _configFrame.tabstrip:GetTab(TAB_FONTS).tabpage
			local checkboxes = tpage.chkFonts
			for i, font in ipairs(myUI_Account.Fonts) do
				checkboxes[i].font = font
				checkboxes[i].fontindex = i
				checkboxes[i].index = i
				checkboxes[i]:SetChecked(defaultFont == font.name)
				if (font.name == defaultFont) then
					checkboxes[i].label:Autosize(font.name .. " " .. L["(default)"])
					_loadFont(checkboxes[i].font)
				else
					checkboxes[i].label:Autosize(font.name)
				end
			end
			_loading = false
		end
		
		function _loadFont(font)
			local fsize = ifnil(font.defaultSize, 14) - 2
			local tpage = _configFrame.tabstrip:GetTab(TAB_FONTS).tabpage
			for _, pv in ipairs(tpage.previews) do
				pv:SetFont(font.src, font.file)
				pv:SetFontSize(fsize)
				pv:SetHeight(pv:GetFullHeight() + 4)
				
				fsize = fsize + 2
			end
		end
		
		local lastAnchor = f
		local xoff = border * 2
		stabpage.chkFonts = {}
		
		for i, font in ipairs(myUI_Account.Fonts) do
			local chk = myUI:CreateFrame("RiftCheckbox", stabpage)
			chk:SetPoint("TOPLEFT", lastAnchor, "BOTTOMLEFT", xoff, 4)
			
			chk.Event.CheckboxChange = _fontChecked
			
			chk.label = myUI:CreateFrame("Text", stabpage)
			chk.label:SetPoint("LEFTCENTER", chk, "RIGHTCENTER", 2, 0)
			
			table.insert(stabpage.chkFonts, chk)
			
			lastAnchor = chk
			xoff = 0
		end
		
		stabpage.previews = {}
		xoff = 220
		lastanchor = stabpage.chkFonts[1]
		lastanchorTo = "TOPRIGHT"
		for i=1, 6, 1 do
			-- we're not creating it via myUI b/c we don't want the font subject to global changes
			local pv = UI.CreateFrame("Text", "myUI__fontPreview", stabpage)
			pv:SetWordwrap(true)		
			pv:SetWidth(230)
			pv:SetBackgroundColor(bgclr:rgba())
			pv:SetPoint("TOPLEFT", lastanchor, lastanchorTo, xoff, 0)
			pv:SetText(L["FontPreview"])
			
			lastanchor = pv
			xoff = 0
			lastanchorTo = "BOTTOMLEFT"
			table.insert(stabpage.previews, pv)
		end
		
		_loadFonts()
		
		local f = myUI:CreateFrame("RiftButton", stabpage)
		f:SetPoint("TOPLEFT", lastAnchor, "BOTTOMLEFT", 0, border)
		f:SetText(L["FontUpdate"])
		f.Event.LeftPress = function(btn)
			local tpage = btn:GetParent()
			for _, chk in ipairs(tpage.chkFonts) do
				if (chk:GetChecked()) then
					myUI:SetGlobalFont(chk.font)
					_loadFonts()
					return
				end
			end		
		end
			
		
		f = myUI:CreateFrame("Text", stabpage)
		f:SetPoint("BOTTOMLEFT", stabpage, "BOTTOMLEFT", border, -border)
		f:SetPoint("BOTTOMRIGHT", stabpage, "BOTTOMRIGHT", -border, border)
		f:SetWordwrap(true)
		f:SetHeight(100)
		f:SetText(L["FontWarning"])
		stabpage.lblFontWarning = f
		
		f = myUI:CreateFrame("Text", stabpage)
		f:SetPoint("BOTTOMLEFT", stabpage.lblFontWarning, "TOPLEFT", 0, -border)
		f:Autosize(L["Font Name"])
		stabpage.lblFontName = f
		
		f = myUI:CreateFrame("RiftTextfield", stabpage)
		f:SetPoint("LEFTCENTER", stabpage.lblFontName, "RIGHTCENTER", 2, 0)
		f:SetWidth(80)
		f:SetBackgroundColor(myUI.color:clone(black, 100):rgba())
		stabpage.txtFontName = f
		
		f = myUI:CreateFrame("Text", stabpage)
		f:SetPoint("LEFTCENTER", stabpage.txtFontName, "RIGHTCENTER", border, 0)
		f:Autosize(L["Font Filename"])
		stabpage.lblFontFile = f
		
		f = myUI:CreateFrame("RiftTextfield", stabpage)
		f:SetPoint("LEFTCENTER", stabpage.lblFontFile, "RIGHTCENTER", 2, 0)
		f:SetWidth(120)
		f:SetBackgroundColor(myUI.color:clone(black,100):rgba())
		stabpage.txtFontFile = f
		
		f = myUI:CreateFrame("RiftButton", stabpage)
		f:SetPoint("LEFTCENTER", stabpage.txtFontFile, "RIGHTCENTER", border, 0)
		f:SetText(L["Add Font"])
		
		f.Event.LeftPress = function(button)
			local stabpage = button:GetParent()
			local trim = myUI.string.trim
			
			local label = trim(stabpage.txtFontName:GetText())
			local fname = trim(stabpage.txtFontFile:GetText())
			
			if (string.len(label) == 0) then
				stabpage.txtFontName:SetKeyFocus(true)
			elseif (string.len(fname) == 0) then
				stabpage.txtFontFile:SetKeyFocus(true)
			else
				table.insert(myUI_Account.Fonts, {src="myUI", name=label, file="fonts/" .. fname})
				stabpage.txtFontName:SetText("")
				stabpage.txtFontName:SetKeyFocus(true)
				stabpage.txtFontFile:SetText("")
			end			
		end
		
		f = myUI:CreateFrame("Text", stabpage)
		f:SetPoint("BOTTOMLEFT", stabpage.lblFontName, "TOPLEFT", 0, 0)
		f:SetText(L["Custom Font"])
		f.Event.MouseIn = function(label)
			myUI:ShowTooltip(label, L["Custom Font Tooltip"])
		end
		f.Event.MouseOut = function(label)
			myUI:HideTooltip(label)
		end
		
		stabpage.lblCustomFont = f
			
		
		end
		
		
		if (true) then -- performance tab
		
		stabpage = _configFrame.tabstrip:GetTab(TAB_PERFORMANCE).tabpage
		
		-- performance monitoring
		
		f = myUI.Widgets.BorderBox:new(stabpage)
		f:SetPoint("TOPLEFT", stabpage, "TOPLEFT", border, border)
		f:SetPoint("BOTTOMRIGHT", stabpage, "BOTTOMRIGHT", -border, - 340)
		stabpage.listCPU = f
		
		local antiBlack = myUI.color:clone(myUI.colors.White, 40)
		local cxyLItem
		
		f = myUI:CreateFrame("Text", stabpage)
		f:SetText(" " .. L["Module ID"])
		f:SetWidth(120)
		cxyLItem = f:GetFullHeight()
		f:SetHeight(cxyLItem)		
		f:SetPoint("TOPLEFT", stabpage.listCPU, "TOPLEFT", 0, 0)
		f:SetBackgroundColor(antiBlack:rgba())
		stabpage.hdrModuleID = f
		
		f = myUI:CreateFrame("Text", stabpage)
		f:SetText(" " .. L["CPU Usage"])
		f:SetWidth(120)
		f:SetHeight(cxyLItem)
		f:SetPoint("TOPLEFT", stabpage.hdrModuleID, "TOPRIGHT", 4, 0)
		f:SetBackgroundColor(antiBlack:rgba())
		stabpage.hdrCPUUsage = f
		
		f = myUI:CreateFrame("Text", stabpage)
		f:SetText(" " .. L["Full Name"])
		f:SetHeight(cxyLItem)
		f:SetPoint("TOPLEFT", stabpage.hdrCPUUsage, "TOPRIGHT", 4, 0)
		f:SetPoint("TOPRIGHT", stabpage.listCPU, "TOPRIGHT", 0, 0)
		f:SetBackgroundColor(antiBlack:rgba())
		stabpage.hdrFullName = f
		
		
		
		local anchors = {stabpage.hdrModuleID, stabpage.hdrCPUUsage, stabpage.hdrFullName}
		local labels = {" myUI", "", " " .. L["myUI Core"]}
		
		stabpage.items = {}
		stabpage.rows = {}
		
		table.insert(stabpage.items, myUI)
		table.insert(stabpage.rows, {})
		for i=1, 3, 1 do
			f = myUI:CreateFrame("Text", stabpage)
			f:SetPoint("TOPLEFT", anchors[i], "BOTTOMLEFT")
			f:SetPoint("TOPRIGHT", anchors[i], "BOTTOMRIGHT")
			f:SetText(labels[i])
			f:SetHeight(cxyLItem)
			
			table.insert(stabpage.rows[1], f)
			
			anchors[i] = f
		end
		
		for _, m in pairs(myUI.Modules) do
			local litem = {
				myUI:CreateFrame("Text", stabpage),
				myUI:CreateFrame("Text", stabpage),
				myUI:CreateFrame("Text", stabpage)
			}
			
			for i=1, #litem, 1 do
				litem[i]:SetPoint("TOPLEFT", anchors[i], "BOTTOMLEFT")
				litem[i]:SetPoint("TOPRIGHT", anchors[i], "BOTTOMRIGHT")
				litem[i]:SetHeight(cxyLItem)
				
				anchors[i] = litem[i]
			end
			
			litem[1]:SetText(" " .. m.Addon)
			litem[2]:SetText("")
			litem[3]:SetText(" " .. m.Name)
			
			table.insert(stabpage.items, m)
			table.insert(stabpage.rows, litem)	

			m.__cpu = -1
		end
		
		_configFrame._lastPerfUpdate = 0
		
		_configFrame.onUpdate = function(self, now, elapsed)
			if (now - _configFrame._lastPerfUpdate) >= 1 then
				_configFrame._lastPerfUpdate = now

				local items = _configFrame.tabstrip:GetTab(TAB_PERFORMANCE).tabpage.items
				local rows = _configFrame.tabstrip:GetTab(TAB_PERFORMANCE).tabpage.rows
				local cpu = Inspect.Addon.Cpu()
				
				local i, j, k, l, age, t, n
				
				for _, m in ipairs(items) do
					m.__cpu = -1
					if (cpu[m.Addon]) then
						local percent = 0
						n = 0
						for kcpu, v in pairs(cpu[m.Addon]) do
							i, j = string.find(v, "%d+%.%d+")
							if (i) then
								t = tonumber(string.sub(v, i, j))
								-- we're interested in current usage, not predictive, so discard times with exponential decay weightings
								if (not string.find(v, "e")) then
									--print(tostring(t))
									
									percent = percent + t									
								end
								--[[
								k, l = string.find(v, "%d+", j)
								--myUI.debug.write(k)
								if (k) then
									-- this reading has exponential falloff so calculate it's weight
									age = tonumber(string.sub(v, k, l))									
									
									t = (1 - math.exp(-1/age)) * t
								else
									--print (t)
									percent = percent + t
									n = n + 1
								end
								]]--
							end
							--[[
							if (m.Addon == "myUI") then
								--myUI.debug.write(kcpu, " = ", v)
							end
							]]--
						end
						m.__cpu = percent * 100
						--if (n ~= 0) then
						--	m.__cpu = percent / n --* 100.0
						--end
					
					end
				end
				
				table.sort(items, function(a, b)
						if (a.__cpu == b.__cpu) then
							return a.Addon < b.Addon
						else
							return a.__cpu > b.__cpu
						end
					end
				)
				
				for i = 1, #items, 1 do
					rows[i][1]:SetText(" " .. items[i].Addon)
					rows[i][3]:SetText(" " .. items[i].Name)
					if (items[i].__cpu >= 0) then
						rows[i][2]:SetText(string.format("%.6f%%", items[i].__cpu))
					else
						rows[i][2]:SetText("??")
					end
				end
			end
		end
		
		-- update speed adj. slider
		
		f = myUI:CreateFrame("Text", stabpage)
		f:Autosize(L["Max Update"])
		f:SetPoint("TOPLEFT", stabpage.listCPU, "BOTTOMLEFT", 0, border)
		stabpage.lblFPS = f
		
		f = myUI:CreateFrame("RiftSlider", stabpage)
		f:SetWidth(200)		
		f:SetPoint("LEFTCENTER", stabpage.lblFPS, "LEFTCENTER", 180, 8)
		f:SetRange(1, 20)
		f:SetPosition(myUI:GetUnitUpdateFPS())
		
		f.Event.SliderGrab = function(slider)
			myUI:ShowTooltip(slider, L["Update/Second"] .. tostring(slider:GetPosition()))
		end
		f.Event.SliderRelease = function(slider)
			myUI:HideTooltip(slider)
			myUI:SetUnitUpdateFPS(slider:GetPosition())
		end
		f.Event.SliderChange = function(slider)
			myUI:ShowTooltip(slider, L["Update/Second"] .. tostring(slider:GetPosition()))
		end
		stabpage.sliderCPU = f
		
		f = myUI:CreateFrame("Text", stabpage)
		f:Autosize(L["Max Update Desc"])
		f:SetPoint("TOPCENTER", stabpage.sliderCPU, "BOTTOMCENTER", 0, -4)
		stabpage.lblFPSDesc = f
				
		-- dynamic update adjustment option
		f = myUI:CreateFrame("RiftCheckbox", stabpage)
		f:SetPoint("TOPLEFT", stabpage.lblFPS, "BOTTOMLEFT", 0, 38)
		f:SetChecked(myUI:GetAdaptiveUpdating())
		f.Event.CheckboxChange = function(chk)
			myUI:SetAdaptiveUpdating(chk:GetChecked())
		end
		stabpage.chkAdaptiveCPU = f
		
		f = myUI:CreateFrame("Text", stabpage)
		f:Autosize(L["Enable Adaptive Updating"])
		f:SetPoint("LEFTCENTER", stabpage.chkAdaptiveCPU, "RIGHTCENTER")
		f.Event.MouseIn = function(label)
			myUI:ShowTooltip(label, L["Adaptive Updating Tooltip"])
		end
		f.Event.MouseOut = function(label)
			myUI:HideTooltip(label)
		end
		stabpage.lblAdaptiveCPU = f
		
		-- animation speed adj. slider
		f = myUI:CreateFrame("Text", stabpage)
		f:Autosize(L["Max Animation"])
		f:SetPoint("TOPLEFT", stabpage.lblFPS, "BOTTOMLEFT", 0, 74)
		stabpage.lblFPS2 = f
		
		f = myUI:CreateFrame("RiftSlider", stabpage)
		f:SetWidth(200)
		f:SetPoint("LEFTCENTER", stabpage.lblFPS2, "LEFTCENTER", 180, 8)
		f:SetRange(5,30)
		f:SetPosition(myUI:GetUpdateFPS())
		
		f.Event.SliderGrab = function(slider)
			myUI:ShowTooltip(slider, L["Update/Second"] .. tostring(slider:GetPosition()))
		end
		f.Event.SliderRelease = function(slider)
			myUI:HideTooltip(slider)
			myUI:SetUpdateFPS(slider:GetPosition())
		end
		f.Event.SliderChange = function(slider)
			myUI:ShowTooltip(slider, L["Update/Second"] .. tostring(slider:GetPosition()))
		end
		stabpage.sliderCPU2 = f
		
		f = myUI:CreateFrame("Text", stabpage)
		f:Autosize(L["Max Animation Desc"])
		f:SetPoint("TOPCENTER", stabpage.sliderCPU2, "BOTTOMCENTER", 0, -4)
		stabpage.lblDFSDesc2 = f
		
		f = myUI:CreateFrame("Text", stabpage)
		f:SetWordwrap(true)
		f:SetText(L["CPU Warning"])
		f:SetHeight(140)
		f:SetPoint("BOTTOMLEFT", stabpage, "BOTTOMLEFT", border, border*2)
		f:SetPoint("BOTTOMRIGHT", stabpage, "BOTTOMRIGHT", -border, border*2)
		
		stabpage.lblCPUWarning = f
		
		f = myUI:CreateFrame("Text", stabpage)
		f:Autosize(L["Simulate Performance"])
		f:SetPoint("BOTTOMLEFT", stabpage.lblCPUWarning, "TOPLEFT", 0, -border)
		stabpage.lblSimulate = f
		
		f = myUI:CreateFrame("RiftButton", stabpage)
		f:SetText(L["Simulate Begin"])
		f:SetPoint("LEFTCENTER", stabpage.lblSimulate, "RIGHTCENTER", border, 0)
		
		
		f.Event.LeftPress = function(btn)
			if (btn._simming) then
				btn:SetText(L["Simulate Begin"])
				btn._simming = false
				
				for _, m in pairs(myUI.Modules) do
					if (m.EndSimulation) then m:EndSimulation() end
				end
			else
				btn:SetText(L["Simulate End"])
				btn._simming = true
				
				for _, m in pairs(myUI.Modules) do
					if (m.BeginSimulation) then m:BeginSimulation() end
				end
			end
		
		end
		f.Event.MouseIn = function(btn)
			myUI:ShowTooltip(btn, L["Simulate Tooltip"])
		end
		f.Event.MouseOut = function(btn)
			myUI:HideTooltip(btn)
		end
		stabpage.btnSimulate = f
		
		end
		
		if (true) then -- lists tab
		
		
		stabpage = _configFrame.tabstrip:GetTab(TAB_LISTS).tabpage
		
		f = myUI:CreateFrame("Text", stabpage)
		f:SetWordwrap(true)
		f:SetText(L["ListsMsg"])
		f:SetPoint("TOPLEFT", stabpage, "TOPLEFT", border, border)
		f:SetPoint("TOPRIGHT", stabpage, "TOPRIGHT", -border, border)
		f:SetHeight(f:GetFullHeight())
		stabpage.lblListMsg = f
		
		f = myUI:CreateFrame("RiftButton", stabpage)
		f:SetText(L["Copy List"])
		f:SetPoint("BOTTOMLEFT", stabpage, "BOTTOMLEFT", border, -border)
		f:SetEnabled(false)
		stabpage.btnCopyList = f
		
		f.Event.LeftPress = function(btnCopyList)
			myUI.inputbox(btnCopyList:GetParent(), L["Copy List Instructions"], _,
				function(result, text, spage)
					if (result == "OK") then
						text = myUI.string.trim(text)
						if (string.find(text, "^[%w%s]+$")) then
							if (myUI.Lists:Exists(text)) then
								myUI.msgbox(spage, L["List Exists"], _, "OK", "INFO")								
							else
								myUI.Lists:CopyList(spage.List:GetSelectedItem(), text) 
								local index = spage.List:AddItem(text)
								spage.List:SetItemSelected(index)		
							end
						else
							myUI.msgbox(spage, L["Bad List Name"], _, "OK", "INFO")
						end
					end
					
				end,
			btnCopyList:GetParent())		
		end
		
		f = myUI:CreateFrame("RiftButton", stabpage)
		f:SetText(L["Delete List"])
		f:SetPoint("BOTTOMLEFT", stabpage.btnCopyList, "TOPLEFT", 0, 6)
		f:SetEnabled(false)
		stabpage.btnDeleteList = f
		
		f.Event.LeftPress = function(btnDeleteList)
			local p = btnDeleteList:GetParent()
			local listname = p.List:GetSelectedItem()
		
			if (listname) then
				myUI.msgbox(btnDeleteList:GetParent(), L["Delete List Confirm"],  _, "YESNO", "QUESTION", 
					function(result, state)
						if (result == "YES") then
							myUI.Lists:Delete(state)
							p.ListItems:ClearItems()
							p.List:RemoveItem(p.List:GetSelectedIndex())
							
							p:__loadList(p.List:GetSelectedIndex())
							return true
						end
					end,
				listname)
			end
		end
		
		
		f = myUI:CreateFrame("RiftButton", stabpage)
		f:SetText(L["Add List"])
		f:SetPoint("BOTTOMLEFT", stabpage.btnDeleteList, "TOPLEFT", 0, 6)
		stabpage.btnAddList = f
		
		stabpage.__loadList = function(spage, selindex)
			spage.ListItems:ClearItems()
			
			if (spage.List:GetItemCount() > 0) and (selindex > 0) then
				local listname = spage.List:GetItem(selindex)
				if (listname) then
					spage.lblListItems:Autosize(L["List Entries:"] .. " " .. listname)
				
					for name, _ in myUI.Lists:ListItemPairs(listname) do
						spage.ListItems:AddItem(name)
					end
					
					spage.btnDeleteList:SetEnabled(true)
					spage.btnCopyList:SetEnabled(true)
					spage.btnAddListItem:SetEnabled(true)
				else
					spage.btnDeleteList:SetEnabled(false)
					spage.btnCopyList:SetEnabled(false)
					spage.btnAddListItem:SetEnabled(false)
				end
			else
				spage.btnDeleteList:SetEnabled(false)
				spage.btnCopyList:SetEnabled(false)
				spage.btnAddListItem:SetEnabled(false)
			end
		end
		
		f.Event.LeftPress = function(btnAddList)
			local spage = btnAddList:GetParent()
			
			myUI.inputbox(spage, L["Add List Instructions"], _, 
				function(result, text, spage)
					if (result == "OK") then
						text = myUI.string.trim(text)
						if (string.find(text, "^[%w%s]+$")) then
							if (myUI.Lists:Exists(text)) then
								myUI.msgbox(spage, L["List Exists"], _, "OK", "INFO")								
							else
								local list = myUI.Lists:Create(text)
								
								local index = spage.List:AddItem(text)
								spage.List:SetItemSelected(index)					
							end
						else
							myUI.msgbox(spage, L["Bad List Name"], _, "OK", "INFO")
						end
					end					
				end,
			spage)	
		end
		
		f = myUI.Widgets.Listbox:new(stabpage)
		
		f:SetPoint("BOTTOMLEFT", stabpage.btnAddList, "TOPLEFT", 0, -12)
		f:SetWidth(220)
		f:SetHeight(280)
		f.SelectedIndexChanged = function(listbox, index)
			local spage = listbox:GetParent()
			spage:__loadList(index)
		end
		
		stabpage.List = f
		
		for lname, list in myUI.Lists:ListPairs() do			
			f:AddItem(lname)
		end
		
		f = myUI:CreateFrame("Text", stabpage)
		f:Autosize(L["Current Lists:"])
		f:SetPoint("BOTTOMLEFT", stabpage.List, "TOPLEFT", 0, -4)
		stabpage.lblCurrentLists = f
		
		f = myUI.Widgets.Listbox:new(stabpage)
		f:SetPoint("TOPLEFT", stabpage.List, "TOPRIGHT", border, 0)
		f:SetPoint("BOTTOMLEFT", stabpage.List, "BOTTOMRIGHT", border, 0)
		f:SetWidth(274)
		f.SelectedIndexChanged = function(listbox, index)
			listbox:GetParent().btnDeleteListItem:SetEnabled(index >= 0)
		end
		stabpage.ListItems = f
		
		f = myUI:CreateFrame("Text", stabpage)
		f:Autosize(L["List Entries:"])
		f:SetPoint("BOTTOMLEFT", stabpage.ListItems, "TOPLEFT", 0, -4)
		stabpage.lblListItems = f
		
		f = myUI:CreateFrame("RiftButton", stabpage)
		f:SetText(L["Add List Entry"])
		f:SetPoint("TOPLEFT", stabpage.ListItems, "BOTTOMLEFT", 0, 12)
		f:SetEnabled(false)
		stabpage.btnAddListItem = f
		
		f.Event.LeftPress = function(btnAddListItem)
			local spage = btnAddListItem:GetParent()
			
			myUI.inputbox(spage, L["Add List Entry Instructions"], _, 
				function(result, text, spage)
					if (result == "OK") then
						text = myUI.string.trim(text)
						if (string.len(text) > 0) then
							local currentList = spage.List:GetSelectedItem()
							if (myUI.Lists:ItemExists(currentList, text)) then
								myUI.msgbox(spage, L["List Entry Exists"], _, "OK", "INFO")								
							else							
								local list = myUI.Lists:AddToList(currentList, text)
								spage.ListItems:AddItem(text)
							end						
						end
					end					
				end,
			spage)
			
		end
		
		f = myUI:CreateFrame("RiftButton", stabpage)
		f:SetText(L["Delete List Entry"])
		f:SetPoint("TOPLEFT", stabpage.btnAddListItem, "BOTTOMLEFT", 0, -6)
		f:SetEnabled(false)
		stabpage.btnDeleteListItem = f
		
		f.Event.LeftPress = function(btnDeleteListItem)
			local p = btnDeleteListItem:GetParent()
			local listname = p.List:GetSelectedItem()
			local itemname = p.ListItems:GetSelectedItem()
			if (listname and itemname) then
				myUI.Lists:RemoveFromList(listname, itemname)
				p.ListItems:RemoveItem(p.ListItems:GetSelectedIndex())				
			end
		
		end
		
		end
	end
	
	return _configFrame
end




function myUI:OnConfigActive()
	--print ("config active")
	
	if (_configFrame) then
		if (not myUI.Events.Update:IsSubscribed("myUI._configFrame.onUpdate")) then
			myUI.Events.UpdateSlow:Subscribe("myUI._configFrame.onUpdate", _configFrame, _configFrame.onUpdate)
		end
	end
end

function myUI:OnConfigInactive()
	--print ("config inactive")
	
	if (_configFrame) then
		if (myUI.Events.Update:IsSubscribed("myUI._configFrame.onUpdate")) then
			myUI.Events.Update:Unsubscribe("myUI._configFrame.onUpdate")
		end
	end
end