local me = {}

local ifnil = myUI.global.ifnil
local isnil = myUI.global.isnil
local notnil = myUI.global.notnil
local iif = myUI.global.iif
local trim = myUI.string.trim

-- returns a table of names for all stored lists
function me:GetListNames()	
	local r = {}
	
	for n, _ in pairs(myUI_Account.Lists) do
		table.insert(r, n)
	end
	
	return r
end

-- returns the actual list. should ONLY be used for list maintenance. Otherwise, use BuildWorkingList()
function me:Get(listname)
	if (notnil(listname)) then
		return myUI_Account.Lists[listname]
	end
end

function me:Create(listname)
	if (isnil(myUI_Account.Lists[listname])) then
		myUI_Account.Lists[listname] = {}
		return myUI_Account.Lists[listname]
	end
end

function me:Delete(listname)
	myUI_Account.Lists[listname] = nil
end

function me:ListPairs() return pairs(myUI_Account.Lists) end
function me:Exists(listname)
	return notnil(myUI_Account.Lists[listname])
end

function me:ItemExists(listname, listitem)
	return ((myUI_Account.Lists[listname]) and (myUI_Account.Lists[listname][listitem]))
end
function me:AddToList(listname, newitem)
	if (myUI_Account.Lists[listname]) then
		if (not myUI_Account.Lists[listname][newitem]) then
			myUI_Account.Lists[listname][newitem] = true
			return true
		end	
	end
end

function me:ListItemPairs(listname)
	if (myUI_Account.Lists[listname]) then
		return pairs(myUI_Account.Lists[listname])
	end
end
function me:RenameList(oldname, newname)
	local items = myUI_Account.Lists[oldname]
	if (items) then
		myUI_Account.Lists[newname] = items
		myUI_Account.Lists[oldname] = nil
		return true
	end
end
function me:RenameListItem(listname, oldname, newname)
	if (myUI_Account.Lists[listname]) then
		myUI_Account.Lists[newname] = true
		myUI_Account.Lists[oldname] = nil
		return true
	end
end
function me:CopyList(sourcelist, newlist)
	if (myUI_Account.Lists[sourcelist]) then
		myUI_Account.Lists[newlist] = myUI.global.shallowCopy(myUI_Account.Lists[sourcelist])
		return true
	end
end

function me:RemoveFromList(listname, itemname)
	if (myUI_Account.Lists[listname]) then
		myUI_Account.Lists[listname][itemname] = nil
	end
end

function me:ClearList(listname)
	if (myUI_Account.Lists[listname]) then 
		myUI_Account.Lists[listname] = {}
	end
end



-- returns a table based on [listnames]. listnames can be a single string, a table of strings, or a comma-seperated list of list names
function me:BuildWorkingList(listnames)
	if (notnil(listnames)) then
		local tofetch
		if (type(listnames) == "string") then
			tofetch = myUI.string.fromCSV(listnames)
		elseif (type(listnames) == "table") then
			tofetch = {}
			for _, lname in ipairs(listnames) do
				table.insert(tofetch, lname)
			end
		end
		
		if (notnil(tofetch)) then
			local r = {}
			
			for _, lname in ipairs(tofetch) do
				lname = trim(lname)
				local acctList = myUI_Account.Lists[lname]
				if (acctList) then
					for k, _ in pairs(acctList) do
						if (not r[k]) then r[k] = true end
					end
				end
			end
			
			return r
		end
	end
end


myUI.Lists = me