local info, data = ...
myUI = {}
myUI.InternalVersion = "0.2.2"
myUI.ReleaseType = "(public beta)"
myUI.Addon = info.identifier --"myUI"
myUI.Name = "myUI Core"

myUI.Context = UI.CreateContext("myUI")
myUI.Context:SetSecureMode("restricted")

myUI.Modules = {}

local L

function myUI:RegisterModule(module)	
	self.Modules[module.Addon] = module
end

local function __subscribeEvent(subscribers, uniqueid, module, func)
	if (nil == func) or (nil == uniqueid) then return false end
	if (nil == subscribers[uniqueid]) then
		subscribers[uniqueid] = {f = func, m = module};
		return true
	else
		return false
	end
end
local function __unsubscribeEvent(subscribers, uniqueid)
	if (nil == uniqueid) then return false end
	subscribers[uniqueid] = nil
	return true
end
local function __isSubscribedEvent(subscribers, uniqueid)
	if (nil == uniqueid) then return false end
	return (nil ~= subscribers[uniqueid])
end
local __eventPrototype = {
	new = function(self)
		local this = {
			_subscribers = {},
			Subscribe = function(self, uniqueid, module, func)
				return __subscribeEvent(self._subscribers, uniqueid, module, func)
			end,
			Unsubscribe = function(self, uniqueid)
				return __unsubscribeEvent(self._subscribers, uniqueid)			
			end,
			IsSubscribed = function(self, uniqueid)
				return __isSubscribedEvent(self._subscribers, uniqueid)
			end
		}
		
		return this
	end
}

myUI.Events = {
	Update = __eventPrototype:new() ,-- function(self, now, elapsed) -- for animations. runs as fast as settings allow (5-20 fps)
	UnitUpdate = __eventPrototype:new(), -- function(self, now, elapsed) -- for unit update polling (5-20 fps)
	UpdateSlow = __eventPrototype:new(), -- function(self, now, elapsed) -- for slower polling options. only fires 2 time a second max
	UnitsChanged = __eventPrototype:new(), -- function(self, units)
	BuffsChanged = __eventPrototype:new(), -- function(self, unit, buffs)
	CombatChanged = __eventPrototype:new(), -- function(self, inCombat)	
	AdaptiveUpdatingChanged = __eventPrototype:new(), -- function(self, adaptiveUpdatingSetting)
}

function myUI:GetAdaptiveUpdating()
	return myUI_Account.AdaptiveUpdate
end
function myUI:SetAdaptiveUpdating(enable)
	if (myUI_Account.AdaptiveUpdate ~= enable) then
		myUI_Account.AdaptiveUpdate = enable
		for _, s in pairs(myUI.Events.AdaptiveUpdatingChanged._subscribers) do
			s.f(s.m, enable)
		end
	end
end

-- This function handles Event.Unit.Available, Event.Unit.Unavailable, Event.Unit.Add, and Event.Unit.Remove, as these
-- are almost always needed as a set
local unitChangers = myUI.Events.UnitsChanged._subscribers
local function __onUnitDetails(units)
	for _, s in pairs(unitChangers) do
		s.f(s.m, units)
	end
end

local buffChangers = myUI.Events.BuffsChanged._subscribers
local function __onBuff(unitID, buffs)
	for _, s in pairs(buffChangers) do
		s.f(s.m, unitID, buffs)
	end
end

local _inCombat = false
local combatChangers = myUI.Events.CombatChanged._subscribers
local function __onCombatEnter()
	_inCombat = true
	--print("combat enter")
	for _, s in pairs(combatChangers) do
		s.f(s.m, true)
	end
end

local function __onCombatLeave()
	--print("combat leave")
	_inCombat = false
	for _, s in pairs (myUI.Events.CombatChanged._subscribers) do
		s.f(s.m, false)
	end
end

function myUI:GetInCombat() return _inCombat end

local updateUnit = 1/5 -- fpx max. available in settings
local updateThrottle = 1/5; --  fps max, will be adjusted to user settings as soon as everything is loaded
local lastUnitUpdate = 0;
local lastUpdate = 0;
local lastSlowUpdate = 0;
local updating = false;

local fastUpdaters = myUI.Events.Update._subscribers
local slowUpdaters = myUI.Events.UpdateSlow._subscribers
local unitUpdaters = myUI.Events.UnitUpdate._subscribers

-- This function is used to provide timer-like functionality using Event.System.Update.Begin
local function __onUpdate()
	if (not updating) then
		updating = true
		local now = Inspect.Time.Frame() 
		local elapsed = now - lastUpdate
		
		if (elapsed >= updateThrottle) then
			lastUpdate = now
			
			for _, s in pairs(fastUpdaters) do
				s.f(s.m, now, elapsed);		
			end			
		end
		
		elapsed = now - lastUnitUpdate
		if (elapsed >= updateUnit) then
			lastUnitUpdate = now
			for _, s in pairs(unitUpdaters) do
				s.f(s.m, now, elapsed)
			end
		end
		
		elapsed = now - lastSlowUpdate
		if (elapsed >= 0.5) then
			-- twice a second update
			lastSlowUpdate = now		
			for _, s in pairs(slowUpdaters) do
				s.f(s.m, now, elapsed)
			end				
		end
		
		updating = false;
	end
end

function myUI:GetUpdateFPS() return math.floor(1/updateThrottle + 0.5) end
function myUI:SetUpdateFPS(fps)
	if (fps) then
		if (fps >= 5) and (fps <= 20) then
			fps = math.floor(fps + 0.5)
			updateThrottle = 1/fps
			myUI_Account.UpdateSpeed = fps
		end
	end
end
function myUI:GetUnitUpdateFPS() return math.floor(1/updateUnit + 0.5) end
function myUI:SetUnitUpdateFPS(fps)
	if (fps) then
		if (fps >= 1) and (fps <= 20) then
			fps = math.floor(fps + 0.5)
			updateUnit = 1/fps
			myUI_Account.UnitUpdateSpeed = fps
		end
	end
end

local function __command(commandline)
	if (commandline == "profile list") or (commandline == "profiles")then
		print "myUI profiles:"
		
		local defp = myUI_Character.CurrentProfile
		for k, _ in pairs(myUI_Account.Profiles) do
			if (k == defp) then
				print("        " .. k .. "    (current)");
			else
				print("        " .. k)
			end
		end
	elseif (string.find(commandline, "profile load ") == 1) then
		local pname = myUI.string.trim(string.sub(commandline, 14))
		if (string.len(pname) > 0) then
			myUI_Character.CurrentProfile = pname			
			print(pname .. L["Profile Activated"])
		end
	elseif (commandline == "version") or (commandline == "ver") then
		print(L["Version "] .. myUI.InternalVersion .. " " .. myUI.ReleaseType);
	elseif (commandline == "modules") or (commandline == "module list") then
		print ("myUI " .. L["modules:"]);
		for module, v in pairs(myUI.CurrentProfile) do
			local mname = module
			if (v.Enabled) then
				print("        " .. mname .. " " .. L["(enabled)"])
			else
				print("        " .. mname)
			end
		end
	--elseif (commandline == "config") then
	--	myUI:ToggleConfig()
	else
		local handled 
		for _, m in pairs(myUI.Modules) do
			if (m.HandleSlashCommand) then
				handled = m:HandleSlashCommand(commandline)
			end
			if (handled) then break end
		end
	
		if (not handled) then
			myUI:ShowUsage()
		end
	end
end

function myUI:ShowUsage()

	local u = self:GetUsage()
	for _, ln in ipairs(u) do
		print(ln)
	end
end

function myUI:GetUsage()
	local r = {}
	
	table.insert(r, L["slash use"])
	table.insert(r, "        profiles, profile list - " .. L["profile list"])
	table.insert(r, "        profile load [" .. L["name"] .. "]* - " .. L["profile load"])
	table.insert(r, "        version, ver - " .. L["version"])
	table.insert(r, "        modules, module list - " .. L["modules"])
		
		
	for _, m in pairs(self.Modules) do
		if (m.GetSlashCommandDescriptions) then
			local lines = m:GetSlashCommandDescriptions()
			if (lines) then
				for _, ln in ipairs(lines) do
					table.insert(r, "        " .. ln)
				end
			end
		end
	end
	
	table.insert(r, "")
	table.insert(r, "                " .. L["reloadui"])
	table.insert(r, "")
	

	return r
end

local function __endLoad(addon)
	if (addon == "myUI") then
		L = myUI:LoadLanguage()
		myUI._L = L
		
		-- create default settings if saved variables have not yet been created
		myUI_Account = myUI:CheckAccountDefaults(myUI_Account) 
		myUI_Character = myUI:CheckCharacterDefaults(myUI_Character)
		
		updateThrottle = 1/myUI_Account.UpdateSpeed
		updateUnit = 1/myUI_Account.UnitUpdateSpeed
		
		-- set current profile based on what the character profile says
		if (not myUI_Account.Profiles[myUI_Character.CurrentProfile]) then
			myUI_Character.CurrentProfile = "default"
		end
		myUI.CurrentProfile = myUI_Account.Profiles[myUI_Character.CurrentProfile];
		
		print(L["myUI Core loaded, Version "] .. myUI.InternalVersion .. " " .. myUI.ReleaseType);

		table.insert(Event.System.Update.End, {__onUpdate,"myUI", "__onUpdate"})	-- is there an advantage of using .End over .Begin? 		
		table.insert(Event.Unit.Remove,{__onUnitDetails, "myUI", "__onUnitDetails"})
		table.insert(Event.Unit.Add,{__onUnitDetails, "myUI", "__onUnitDetails"})
		table.insert(Event.Unit.Available,{__onUnitDetails, "myUI", "__onUnitDetails"})
		table.insert(Event.Unit.Unavailable,{__onUnitDetails, "myUI", "__onUnitDetails"})
		
		table.insert(Event.Buff.Add, {__onBuff, "myUI", "__onBuff"})
		--table.insert(Event.Buff.Change, {__onBuff, "myUI", "__onBuff"})
		table.insert(Event.Buff.Remove, {__onBuff, "myUI", "__onBuff"})
		
		table.insert(Event.System.Secure.Enter, {__onCombatEnter, "myUI", "__onCombatEnter"})
		table.insert(Event.System.Secure.Leave, {__onCombatLeave, "myUI", "__onCombatLeave"})
		
		table.insert(Command.Slash.Register("myui"), {__command, "myUI", "__command"});
		
	end
end

local function __allLoaded()
	
	local profile = myUI.CurrentProfile
	
	for _, module in pairs(myUI.Modules) do
		
		if (module.SettingsKey) and (module.GetDefaults) then
			if (profile[module.SettingsKey] == nil) then				
				profile[module.SettingsKey] = module:GetDefaults()				
			end
		end
		
		if (module.SettingsKey) and (not myUI_Character[module.SettingsKey]) then
			myUI_Character[module.SettingsKey] = {}
		end
		
		if (module.Initialize) then module:Initialize(profile, myUI_Character) end
		local mname = module.Name
		local mrel = module.ReleaseType
		local mver = module.ModuleVersion
		print (mname .. " " .. L[" loaded, Version "] .. mver .. " " .. mrel);		
	end
	
	-- give modules a chance to hook after all modules are initialized
	for _, module in pairs(myUI.Modules) do
		if (module.PostInitialize) then module:PostInitialize() end
	end
end

local function __varsLoaded(addon)
	if (addon == "myUI") then
		--print("Saved variables loaded")
		
	end
end


table.insert(Event.Addon.Load.End, {__endLoad, "myUI", "__endLoad"})
table.insert(Event.Addon.SavedVariables.Load.End, {__varsLoaded, "myUI", "__varsLoaded"})
table.insert(Event.Addon.Startup.End, {__allLoaded, "myUI", "__allLoaded"})


-- Setup frame recycling 

-- NOTE: a consequence of recycling frames is that recycled frames may have some members set, so whenever calling myUI:CreateFrame() ALWAYS
--	initalize any members for the new frame.

-- NOTE: any created frame which contains child frames should override its Dispose() method to make sure of disposing it's child frames as well.

myUI._frameCount = 0;

myUI._recycled = {
	["Frame"]={},
	["Text"] = {},
	["Texture"] = {},
	["RiftButton"] = {},
	["RiftCheckbox"] = {},
	["RiftSlider"] = {},
	["RiftTextfield"] = {},
	["RiftWindow"] = {},
	["Mask"] = {}
}
myUI._text = {}	-- reference to all text frames created so that we can set/update global fonts

-- utility function to be appened to all text frames created
local function _autosize(textframe, text)
	if (nil ~= textframe) then
		if (nil ~= text) then textframe:SetText(text) end
	
		--textframe:SetWidth(textframe:GetFullWidth())
		--textframe:SetHeight(textframe:GetFullHeight())
	end
end

-- utility function to resize in one call for all frames
local function _setSize(frame, cx, cy)
	if (frame) then
		if (cx) then frame:SetWidth(cx) end
		if (cy) then 
			frame:SetHeight(cy) 
		elseif (cx) then
			frame:SetHeight(cx)
		end
	end
end
local function _getSize(frame)
	if (frame) then
		local x = frame:GetWidth()
		local y = frame:GetHeight()
		
		return x, y
	end
end
-- utility function to get the top, non-UIParent of a frame
local function _getTopParent(frame)
	local tparent, parent
	
	parent = frame:GetParent()
	while (p ~= nil) and (p ~= UIParent) do
		tparent = p
		p = tparent:GetParent()	
	end

	return tparent
end

function myUI:CreateFrame(ftype, parent, alpha, visible, width, height)
	local ifnil = myUI.global.ifnil
	local notnil = myUI.global.notnil
	
	parent = ifnil(parent, myUI.Context)
	alpha = ifnil(alpha, 1)
	visible = ifnil(visible, true);
	
	local f;
	
	if (#self._recycled[ftype] > 0) then
		f = table.remove(self._recycled[ftype]);
		
		f:SetParent(parent);		
	else
		self._frameCount = self._frameCount + 1;
		local name = "myUI." .. ftype .. "." .. self._frameCount;
		f = UI.CreateFrame(ftype, name, parent);
		
		if (ftype == "Text")  then table.insert(self._text, f) end
	end
	
	-- add some extra methods to myUI-created frames
	f.SetSize = _setSize
	f.GetSize = _getSize
	f.GetTopParent = _getTopParent
	
	-- add a Dispose support.
	f.Dispose = myUI.RecycleFrame(this);		
	
	f.IsDisposed = false;
	
	f:ClearAll();
	
	if (notnil(width)) then f:SetWidth(width) end
	if (notnil(height)) then f:SetHeight(height) end
	
	if (ftype == "Text") then
		if (self.CurrentProfile.globalFont) then
			f:SetFont(self.CurrentProfile.globalFont.src, self.CurrentProfile.globalFont.file)		
			f:SetFontSize(ifnil(self.CurrentProfile.globalFont.defaultSize, 14))
		else
			f:SetFontSize(14)						
		end
		f:SetFontColor(1,1,1,1)
		f.Autosize = _autosize
		f:SetWordwrap(false)
	elseif (ftype == "RiftCheckbox") then
		f:ResizeToDefault()
	end
	
	f:SetVisible(visible);	
	f:SetAlpha(alpha)
	return f
end



function myUI:SetGlobalFont(font)
	if (not font) then return end
	
	for _, f in ipairs(self._text) do		
		f:SetFont(font.src, font.file)	
		--print("set font for " .. f:GetName())
	end
	
	myUI.CurrentProfile.globalFont = {src = font.src, file = font.file, name = font.name, defaultSize = font.defaultSize}
end

function myUI:RecycleFrame(frame)
	if (nil ~= frame) then
		frame.IsDiposed = true
		frame.Dispose = nil -- in case default dispose function was overridden
		
		local name = frame:GetName();
		local ftype = "Frame"
		
		frame:ClearAll();
		frame:SetVisible(false);
		frame:SetLayer(0)
		frame:SetAlpha(0)
		frame:SetBackgroundColor(myUI.colors.Transparent:rgba())
		frame:SetParent(self.Context);
		
		if (string.find(name, "myUI.Texture")) then
			ftype = "Texture"
		elseif (string.find(name, "myUI.Text")) then
			ftype = "Text"
			frame:SetText("");		
		elseif (string.find(name, "myUI.RiftButton")) then
			ftype = "RiftButton"
		elseif (string.find(name, "myUI.RiftCheckbox")) then
			ftype = "RiftCheckbox"
		elseif (string.find(name, "myUI.RiftSlider")) then
			ftype = "RiftSlider"
		elseif (string.find(name, "myUI.RiftTextfield")) then
			ftype = "RiftTextfield"
		elseif (string.find(name, "myUI.RiftWindow")) then
			ftype = "RiftWindow"
		end
		
		table.insert(self._recycled[ftype], frame);
	end
end




