local iif = myUI.global.iif
local ifnil = myUI.global.ifnil
local isnil = myUI.global.isnil
local notnil = myUI.global.notnil
local minmax = myUI.global.minmax

local stringex = {}

function stringex.trim(s)return (s:gsub("^%s*(.-)%s*$", "%1"))end
function stringex.ltrim(s) return (s:gsub("^%s*", "")) end
function stringex.rtrim(s)
    local n = #s
    while n > 0 and s:find("^%s", n) do n = n - 1 end
    return s:sub(1, n)
end

function stringex.lpad(s, len, pchar)
    pchar = ifnil(pchar, " ");
    if (string.len(pchar) > 1) then pchar = string.sub(pchar,1,1); end

    s = string.rep(pchar, len) .. s;    
    return string.sub(s, -len);
end

function stringex.rpad(s, len, pchar)
    pchar = ifnil(pchar, " ");
    if (string.len(pchar) > 1) then pchar = string.sub(pchar,1,1); end

    s = s .. string.rep(pchar, len);
    return string.sub(s, 1, len);
end
   
function stringex.tohex(n)
    if (n ~= nil) then
        n = tonumber(n);        
        return stringex.lpad(string.format("%x", n), 2, "0");
    else
        return "nil";
    end
end

function stringex.nilstr(s)
    return tostring(ifnil(s, "nil"));
end

function stringex.isempty(s)
    if (isnil(s)) then
        return true;
    elseif (type(s) == "string") then            
        return not (string.len(stringex.trim(s)) > 0);            
    else
        return not (string.len(tostring(s)) > 0);
    end                     
end
        
function stringex.getTimeString(t, short)
	if (t) then
		if (short) then
			if (t <= 0) then
				t = ""
			elseif (t <= 60) then
				t = tostring(math.ceil(t)) .. "s" --string.format("%02d", t)
			elseif (t >= 3600) then
				t = string.format("%.1f", t / 3600) .. "h"
			else
				t = string.format("%02d", t/60) .. "m"
			end		
		else			
			if (t <= 10) then
				t = string.format("%.1f", t)
			elseif (t <= 60) then
				t = string.format("%02d", t)
			elseif (t >= 3600) then
				t = string.format("%d:%02d", t / 3600, t / 60 % 60)
			else
				t = string.format("%d:%02d", t / 60, t % 60)
			end
		end
		
		return t
	else
		return ""
	end
end	
	
    -- Convert from CSV string to table
function stringex.fromCSV(s)
    s = s .. ','        -- ending comma
    local t = {}        -- table to collect fields
    local fieldstart = 1
    repeat
        -- next field is quoted? (start with `"'?)
        if string.find(s, '^"', fieldstart) then
        local a, c
        local i  = fieldstart
        repeat
            -- find closing quote
            a, i, c = string.find(s, '"("?)', i+1)
        until c ~= '"'    -- quote not followed by quote?
        if not i then error('unmatched "') end
        local f = string.sub(s, fieldstart+1, i-1)
        table.insert(t, (string.gsub(f, '""', '"')))
        fieldstart = string.find(s, ',', i) + 1
        else                -- unquoted; find next comma
        local nexti = string.find(s, ',', fieldstart)
        table.insert(t, string.sub(s, fieldstart, nexti-1))
        fieldstart = nexti + 1
        end
    until fieldstart > string.len(s)
    return t
end  
    
myUI.string = stringex