local me = {}

local ifnil = myUI.global.ifnil
local isnil = myUI.global.isnil
local notnil = myUI.global.notnil
local iif = myUI.global.iif
local trim = myUI.string.trim
local colors = myUI.colors

local function __appliesToStyle(self, frame)
	return (notnil(frame) and self.applies[type(frame)])
end

local function __applyFrameStyle(self, frame)
	if (notnil(frame) and self:AppliesTo(frame)) then
		if (self.bgcolor) then frame:SetBackgroundColor(self.bgcolor:rgba()) end
		if (self.alpha) then frame:SetAlpha(self.alpha) end
		if (self.layer) then frame:SetLayer(self.layer) end
		if (self.width) then frame:SetWidth(self.width) end
		if (self.height) then frame:SetHeight(self.height) end
		if (notnil(self.visible)) then frame:SetVisible(self.visible) end
		return true
	else
		return false
	end

end


local framestyle = {
	new = function(self, bgcolor, alpha, layer, width, height, visible)
		local this = {}
		this.bgcolor = bgcolor
		this.alpha = alpha
		this.layer = layer
		this.width = width
		this.height = height
		this.visible = visible
		this.applies = {["Frame"]=true, ["Text"]=true, ["Texture"]=true, ["RiftButton"]=true, ["RiftCheckbox"]=true, ["RiftSlider"]=true, ["RiftTextfield"]=true, ["RiftWindow"]=true}
	
	
		this.Apply = __applyFrameStyle
		this.AppliesTo = __appliesToStyle
		
		return this
	end
}
me.FrameStyle = framestyle

local function __applyTextStyle(self, frame)
	if (__applyFrameStyle(self, frame)) then
		frame:SetFontColor(self.fgcolor:rgba())
		frame:SetWordwrap(self.wrap)
		
		if (notnil(self.fontsrc) and notnil(self.fontname)) then
			frame:SetFont(self.fontsrc, self.fontname)
		end
		frame:SetFontSize(self.fontsize)
		
		return true
	else
		return false
	end

end


local textstyle = {
	new = function(self, fontsrc, fontname, fontsize, fgcolor, wrap, bgcolor, alpha, layer, width, height, visible)
		local this = framestyle:new(bgcolor, alpha, layer, width, height, visible)
	
		this.fgcolor = ifnil(fgcolor, colors.White)
		this.wrap = ifnil(wrap, false)
	
		local gfont = myUI.CurrentProfile.globalFont
		if (isnil(fontsrc) and isnil(fontname) and isnil(fontsize)) then		
			this.fontsrc = gfont.src
			this.fontname = gfont.file
			this.fontsize = ifnil(gfont.defaultSize, 14)
		else
			this.fontsrc = fontsrc
			this.fontname = fontname
			this.fontsize = fontsize
		end
	
		this.applies = {["Text"]=true}
		
		this.Apply = __applyTextStyle
		
		return this
	end
}
me.TextStyle = textstyle

local function __applyTextureStyle(self, frame)
	if (__applyFrameStyle(self, frame)) then
		if (notnil(self.texturesrc) and notnil(self.texturefile)) then
			frame:SetTexture(self.texturesrc, self.texturefile)
		end
	
		return true
	else
		return false
	end

end

local texturestyle = {
	new = function(self, texturesrc, texturefile, bgcolor, alpha, layer, width, height, visible)
		this = framestyle:new(bgcolor, alpha, layer, width, height, visible)
		
		this.texturesrc = texturesrc
		this.texturefile = texturefile
	
		this.applies = {["Texture"] = true}
		this.Apply = __applyTextureStyle
		
		return this
	end
}

me.TextureStyle = texturestyle

myUI.Styles = me

