nkGFX.context = UI.CreateContext("HUD")


function nkGFX.Tools:HexToRGB(hexValue)

	if type(hexValue) ~= "string" then return nil end
	if string.len(hexValue) < 6 then return nil end
	
	local retValue = {}
	
	for i = 1, 6, 2 do		
		local subString = string.sub(hexValue, i, (i+1))
		local decValue = tonumber(subString, 16)
		table.insert (retValue, (1 / 255 * decValue))
	end
	
	return retValue
	
end

function nkGFX.Tools:DecToHex(value)

    local b,k,result,i,d=16,"0123456789ABCDEF","",0
	
    while value > 0 do
        i=i+1
        value,d = math.floor(value/b),math.mod(value,b)+1
        result=string.sub(k,d,d)..result
    end
	
    return result
	
end

function nkGFX.Tools:ColorAdjust(color, adjust)

	if type(color) ~= "string" then return nil end
	if string.len(color) < 6 then return nil end
	local newColor = ''
	
	for i = 1, 6, 2 do		
		local subString = string.sub(color, i, (i+1))
		local decValue = tonumber(subString, 16)
		decValue = decValue * adjust
		local hex = nkGFX.Tools:DecToHex(decValue)
		if string.len(hex) == 1 then hex = '0' .. hex end			
		newColor = newColor .. hex
	end
	
	return newColor

end

function nkGFX.Button:new (buttonData)
	
	if buttonData == nil then return nil end
	local data = buttonData
	
	if data.template ~= nil then
		for k, value in pairs (data.template) do
			data[k] = value
		end
	end
	
	local width = 5
	if data.width ~= nil then width = data.width end
	
	local button = nkGFX.Frame:new({id = data.id, parent = data.parent, width = width, height = 25, color= {border = data.color.border, body = data.color.body}, anchors = data.anchors })
	
	local tempColor = nkGFX.Tools:ColorAdjust(data.color.body, 1.2)

	button.upper = nkGFX.Frame:new({id = data.id .. 'upper', parent = button:GetBody(true), height = 12, color= {body = tempColor}, anchors = {{from = "TOPLEFT", object = button:GetBody(true), to = "TOPLEFT"},{ from = "TOPRIGHT", object = button:GetBody(true), to = "TOPRIGHT"} }})	
	button.label = nkGFX.Text:new({id = data.id .. 'label', text = data.label, fontsize = 14, parent = button.upper:GetBody(), color = data.color.label , anchors = {{ from = "CENTER", object = button:GetBody(), to = "CENTER"}}})

	if data.width == nil or data.autoWidth == true then 
		local width = button.label:GetWidth(true)
		button:SetWidth (width+10)
	end

	if data.func ~= nil then
		local frame = button:GetBody(false)
		function frame.Event:LeftClick() buttonData.func() end
	end
	
	data.frame = button
	
	setmetatable(data, self)
	self.__index = self
	
	return data
	
end

function nkGFX.Button:GetBody(flag)

	return self.frame:GetBody(flag);

end

function nkGFX.Button:GetWidth ()

	return self.frame:GetWidth()

end

function nkGFX.Frame:new(frameData)

	if frameData == nil then return nil end

	local frame = UI.CreateFrame("Frame", frameData.id, frameData.parent)
	
	if frameData.width ~= nil then frame:SetWidth(frameData.width)  end
	if frameData.height ~= nil then frame:SetHeight(frameData.height) end
	
	if frameData.anchors ~= nil then
		for k, anchor in pairs(frameData.anchors) do
			if anchor.x ~= nil and anchor.y ~= nil then
				frame:SetPoint (anchor.from, anchor.object, anchor.to, anchor.x, anchor.y)
			else
				frame:SetPoint (anchor.from, anchor.object, anchor.to)
			end
		end
	end
	
	if frameData.callBack ~= nil then
		function frame.Event:LeftClick()
			frameData.callBack(self)
		end
	end
	
	if frameData.color ~= nil then	
		if frameData.color.body ~= nil then
			if frameData.color.border ~= nil then
				local alpha = frameData.color.alphaborder or 1
				local color = nkGFX.Tools:HexToRGB(frameData.color.border)
				frame:SetBackgroundColor(color[1], color[2], color[3], alpha)
				frame.body = nkGFX.Frame:new({id = frameData.id .. 'body', parent = frame, color = { body = frameData.color.body, alphabody = frameData.color.alphabody }, anchors = {{from = "TOPLEFT", object = frame, to = "TOPLEFT", x = 1, y = 1}, {from = "BOTTOMRIGHT", object = frame, to ="BOTTOMRIGHT", x = -1, y = -1}}})	
			else
				local color = nkGFX.Tools:HexToRGB(frameData.color.body)
				local alpha = frameData.color.alphabody or 1
				frame:SetBackgroundColor(color[1], color[2], color[3], alpha)
			end
		end
	end	
	
	if frameData.visible == false then frame:SetVisible(false) end
	
	local data = frameData
	frameData.frame = frame
	
	setmetatable(data, self)
	self.__index = self
	
	return data
end

function nkGFX.Frame:GetBody(flag)

	if self.frame.body == nil or flag == false then
		return self.frame
	else
		return self.frame.body.frame
	end

end

function nkGFX.Frame:SetVisible(flag)

	self.frame:SetVisible(flag)
	
end

function nkGFX.Frame:SetLayer(layer)

	self:GetBody(false):SetLayer(layer)

end

function nkGFX.Frame:SetColor (colorData)

	if colorData == nil then return end
	
	if colorData.body ~= nil then
		if self.frame.body == nil then
			local color = nkGFX.Tools:HexToRGB(colorData.body)
			local alpha = colorData.bodyAlpha or 1
			self.frame:SetBackgroundColor(color[1], color[2], color[3], alpha)
		else
			self.frame.body:SetColor(colorData)
		end
	end
	
	if colorData.border ~= nil and self.frame.body ~= nil then
		local color = nkGFX.Tools:HexToRGB(frameData.color.border)
		local alpha = colorData.borderAlpha or 1
		self.frame:SetBackgroundColor(color[1], color[2], color[3], alpha)
	end

end

function nkGFX.Frame:GetWidth ()

	return self.frame:GetWidth()

end

function nkGFX.Frame:SetHeight(height)

	if height == nil then return end	
	self.frame:SetHeight(height)

end

function nkGFX.Frame:SetWidth(width)

	if width == nil then return end	
	self.frame:SetWidth(width)

end

function nkGFX.Frame:Update (settings)

	if settings == nil then return end
	for k, v in pairs (settings) do 
		if k == 'anchors' then
			self.frame:ClearAll()
			self[k] = v
		elseif k == 'color' and self.color ~= nil then
			for k2, v2 in pairs (v) do
				self.color[k2] = v2
			end
		else
			self[k] = v 
		end
	end
		
	if self.width ~= nil then 
		self.frame:SetWidth(self.width) 
		if self.frame.body ~= nil then self.frame.body:Update({ width = self.width -2 } ) end
	end
	if self.height ~= nil then 
		self.frame:SetHeight(self.height) 
		if self.frame.body ~= nil then 
			self.frame.body:Update({ height = self.height -2 }) 
		end
	end

	if self.color ~= nil then	
		if self.color.body ~= nil then
			if self.color.border ~= nil then
				local alpha = self.color.alphaborder or 1
				local color = nkGFX.Tools:HexToRGB(self.color.border)
				self.frame:SetBackgroundColor(color[1], color[2], color[3], alpha)
				self.frame.body:Update({ color = { body = self.color.body, alphabody = self.color.alphabody } })
			else
				local color = nkGFX.Tools:HexToRGB(self.color.body)
				local alpha = self.color.alphabody or 1
				
				self.frame:SetBackgroundColor(color[1], color[2], color[3], alpha)
			end
		end
	end	
	
	if self.anchors ~= nil then
		for k, anchor in pairs(self.anchors) do
			if anchor.x ~= nil and anchor.y ~= nil then
				self.frame:SetPoint (anchor.from, anchor.object, anchor.to, anchor.x, anchor.y)
			else
				self.frame:SetPoint (anchor.from, anchor.object, anchor.to)
			end
		end
	end
	
end

function nkGFX.Frame:SetEvent(event, callback)

	local body = self:GetBody()

	if event == "LeftClick" then
		function body.Event:LeftClick() callback(body)  end
	elseif event == "LeftDown" then
		function body.Event:LeftDown() callback(body)  end
	elseif event == "MouseMove" then
		function body.Event:MouseMove() callback(body)  end
	elseif event == "LeftUp" then
		function body.Event:LeftUp() callback(body)  end
	end

end

function nkGFX.Text:new (textData)

	if textData == nil then return end
	local data = textData
	
	if data.template ~= nil then
		for k, value in pairs (data.template) do
			if data[k] == nil then data[k] = value end
		end
	end
	
	local textFrame = UI.CreateFrame("Text", data.id, data.parent)
	if data.text ~= nil then textFrame:SetText (data.text) end
	if data.fontsize ~= nil then textFrame:SetFontSize(data.fontsize) end
	if data.wordwrap ~= nil then textFrame:SetWordwrap(data.wordwrap) end
	
	if data.height == nil then 
		textFrame:SetHeight(textFrame:GetFullHeight())
	else
		textFrame:SetHeight(data.height)	
	end
		
	
	if data.width == nil or (data.autowidth == true and textFrame:GetFullWidth() < data.width) then
		textFrame:SetWidth(textFrame:GetFullWidth())	
	else
		textFrame:SetWidth(data.width)
	end	
	
	local alpha = data.alpha or 1
	local color = nkGFX.Tools:HexToRGB (data.color)

	textFrame:SetFontColor(color[1], color[2], color[3], alpha)
	
	if data.anchors ~= nil then
		for k, anchor in pairs(data.anchors) do
			if anchor.x ~= nil and anchor.y ~= nil then
				textFrame:SetPoint (anchor.from, anchor.object, anchor.to, anchor.x, anchor.y)
			else
				textFrame:SetPoint (anchor.from, anchor.object, anchor.to)
			end
		end	
	end
		
	data.frame = textFrame
	
	setmetatable(data, self)
	self.__index = self
	return data

end

function nkGFX.Text:SetText(text)

	if text == nil then return end	
	self.frame:SetText(text)
	self.text = text
	
	if self.width == nil or (self.autowidth == true and self.frame:GetFullWidth() < self.width) then
		self.frame:SetWidth(self.frame:GetFullWidth())	
	else
		self.frame:SetWidth(self.width)
	end
	
	if self.height == nil then
		self.frame:SetHeight(self.frame:GetFullHeight())
	else
		self.frame:SetHeight(self.height)
	end
	
	
end

function nkGFX.Text:Update(settings)

	if settings == nil then return end
	for k, v in pairs (settings) do self[k] = v end
	
	self.frame:SetText (self.text)
	if self.wordwrap ~= nil then self.frame:SetWordwrap(self.wordwrap) end
	
	if self.width == nil or (self.autowidth == true and self.frame:GetFullWidth() < self.width) then
		self.frame:SetWidth(self.frame:GetFullWidth())			
	else
		self.frame:SetWidth(self.width)
	end	
	
	self.frame:SetFontSize(self.fontsize)
	
	if self.anchors ~= nil then
		for k, anchor in pairs(self.anchors) do
			if anchor.x ~= nil and anchor.y ~= nil then
				self.frame:SetPoint (anchor.from, anchor.object, anchor.to, anchor.x, anchor.y)
			else
				self.frame:SetPoint (anchor.from, anchor.object, anchor.to)
			end
		end	
	end	
	
	local alpha = self.alpha or 1
	local color = nkGFX.Tools:HexToRGB (self.color)
	
	self.frame:SetFontColor(color[1], color[2], color[3], alpha)

end

function nkGFX.Text:GetWidth(flag)

	if flag == true then
		return self.frame:GetFullWidth()
	else
		return self.frame:GetWidth()
	end

end

function nkGFX.Text:SetVisible(flag)

	if flag == nil then return end
	
	self.frame:SetVisible(flag)

end

function nkGFX.TabPane:new (tabPaneData)

	if tabPaneData == nil then return end
	local data = tabPaneData
	data.activePane = 1
	data.color = tabPaneData.color
	
	local tabPane = nkGFX.Frame:new({id = tabPaneData.id, parent = tabPaneData.parent, width = tabPaneData.width, height = tabPaneData.height, anchors = tabPaneData.anchors})
			
	tabPane.body = nkGFX.Frame:new({id = tabPaneData.id .. 'body', color = {border = tabPaneData.color.border, body = tabPaneData.color.body}, parent = tabPane:GetBody(), width = tabPaneData.width, height = (tabPaneData.height-29), anchors = {{from = "BOTTOMLEFT", object = tabPane:GetBody(), to ="BOTTOMLEFT"}}})	

	local lastObject = tabPane:GetBody()
	local lastObjectTo = "TOPLEFT"
	local index = 1

	tabPane.header = {}
	
	for k, pane in pairs(tabPaneData.panes) do	
		pane.content:SetVisible(false)
		pane.content:SetLayer(1)
	
		local header = nkGFX.Frame:new({id = tabPaneData.id .. 'header' .. k, color = {border = tabPaneData.color.border, body = tabPaneData.color.inactive}, parent = tabPane.body:GetBody(), width = 150, height = 30, anchors = {{from = "TOPLEFT", object = lastObject, to = lastObjectTo }}})	
		header.label = nkGFX.Text:new({id = tabPaneData.id .. 'header' .. k .. 'Label', fontsize = 14, text = pane.label, color = tabPaneData.color.label, parent = header:GetBody(), anchors = {{from = "CENTER", object = header:GetBody(), to = "CENTER" , x = -1, y = 0}} })
		header.line = nkGFX.Frame:new({id = tabPaneData.id .. 'header' .. k .. 'Line', color = {body = tabPaneData.color.body}, parent = header:GetBody(false), width = 148, height = 1, anchors = {{from = "BOTTOMLEFT", object = header:GetBody(false), to = "BOTTOMLEFT", x=1, y=0 }}})	
		header.line:SetVisible(false)
		
		pane.content:GetBody():SetPoint("TOPLEFT", tabPane.body:GetBody(), "TOPLEFT")
		pane.content:GetBody():SetPoint("BOTTOMRIGHT", tabPane.body:GetBody(), "BOTTOMRIGHT")
		
		local body = header:GetBody()
		body.paneNo = index
		
		function body.Event:LeftClick()
			tabPane.header[data.activePane]:SetColor({body = data.color.inactive})
			tabPane.header[data.activePane].line:SetVisible(false)
			data.panes[data.activePane].content:SetVisible(false)			
			data.activePane=self.paneNo
			tabPane.header[data.activePane]:SetColor({body = data.color.body})
			tabPane.header[data.activePane].line:SetVisible(true)
			data.panes[data.activePane].content:SetVisible(true)
		end
		
		lastObjectTo = "TOPRIGHT"
		lastObject = header:GetBody(false)
	
		table.insert(tabPane.header, header)
		index = index + 1
	end
	
	tabPane.header[1].line:SetVisible(true)
	tabPane.header[1]:SetColor({body = data.color.body})
	data.panes[1].content:SetVisible(true)
	
	data.frame = tabPane
	
	setmetatable(data, self)
	self.__index = self
	return data

end

function nkGFX.TabPane:GetBody ()

	return self.frame.body:GetBody()

end

function nkGFX.TabPane:switchTo (pane)

	print (pane)
--[[
	self.tabHeader[self.activePane].body:SetBackgroundColor(self.color.inactive)
	self.tabHeader[self.activePane].body:SetHeight(30)
	
	self.activePane = pane
	
	self.tabHeader[self.activePane].body:SetBackgroundColor(self.color.body)
	self.tabHeader[self.activePane].body:SetHeight(31)
	]]
end

function nkGFX.Window:new(windowData)
	
	if windowData == nil then return nil end
	local data = windowData
	
	if data.template ~= nil then
		for k, value in pairs (data.template) do
			data[k] = value
		end
	end
	
	if data.collapsed == nil then data.collapsed = false end	
	
	if data.parent == nil then data.parent = nkGFX.context end
	local window = nkGFX.Frame:new({id = data.id, parent = data.parent, width = data.width, height = data.height, anchors = data.anchors})	

	function window.frame.Event:LeftClick() end
	
	window.header = nkGFX.Frame:new({id = data.id .. 'header', parent = window.frame, height = 30, color = {border = data.color.border, body = data.color.header}, anchors = {{from = "TOPLEFT", object = window.frame, to = "TOPLEFT"}, {from = "TOPRIGHT", object = window.frame, to ="TOPRIGHT"}} })
	window.headerTitle = nkGFX.Text:new({id = data.id .. 'headerTitle', text = data.title, fontsize = 14, parent = window.header:GetBody(), color = data.color.title, anchors = {{ from = "CENTERLEFT", object = window.header:GetBody(), to = "CENTERLEFT", x = 10, y = 0}}})

	local collapseChar = '\94'
	if data.collapsed == true then collapseChar = 'v' end

	if data.closeable == true then
		window.closeBox = nkGFX.Frame:new({id = data.id .. 'close', parent = window.header:GetBody(), width = 20, height = 20, color = { border = data.color.border, body = data.color.header}, anchors = {{from = "TOPRIGHT", object = window.header:GetBody(), to = "TOPRIGHT", x=-4,y=4}}})
		window.closeBox.X = nkGFX.Text:new({id = data.id .. 'closeX', text = "X", fontsize = 14, parent = window.closeBox:GetBody(), color = data.color.border, anchors = {{ from = "CENTER", object = window.closeBox:GetBody(), to = "CENTER"}}})
		
		if windowData.closeFunc ~= nil then
			window.closeBox:SetEvent('LeftClick', function() window:SetVisible(false); windowData.closeFunc() end )
		else
			window.closeBox:SetEvent('LeftClick', function() window:SetVisible(false) end )
		end
	end
	
	if data.footer ~= nil then
		window.footer = nkGFX.Frame:new({id = data.id .. 'footer', parent = window.frame, width = data.width, height = 30, color = {border = data.color.border, body = data.color.footer}, anchors = {{from = "BOTTOMLEFT", object = window.frame, to = "BOTTOMLEFT"}} })
		window.footerTitle = nkGFX.Text:new({id = data.id .. 'footerTitle', text = data.footer, fontsize = 14, parent = window.footer:GetBody(), color = data.color.title, anchors = {{ from = "CENTERLEFT", object = window.footer:GetBody(), to = "CENTERLEFT", x = 10, y = 0}}})

		window.body = nkGFX.Frame:new({id = data.id .. 'body', parent = window:GetBody(false), color = {border = data.color.border, body = data.color.body}, anchors={{from = "TOPLEFT", object = window.header:GetBody(false), to = "BOTTOMLEFT", x = 0, y = -1}, {from = "BOTTOMRIGHT", object = window.footer:GetBody(false), to = "TOPRIGHT", x=0, y=1}}})
	else
		window.body = nkGFX.Frame:new({id = data.id .. 'body', parent = window:GetBody(false), color = {border = data.color.border, body = data.color.body}, anchors={{from = "TOPLEFT", object = window.header:GetBody(false), to = "BOTTOMLEFT", x = 0, y = -1}, {from = "BOTTOMRIGHT", object = window:GetBody(), to = "BOTTOMRIGHT"}}})
	end
	
	if data.collapsible == true then
		if data.closeable == true then
			window.collapseBox = nkGFX.Frame:new({id = windowData.id .. 'collapse', parent = window.header:GetBody(), width = 20, height = 20, color = { border = windowData.color.border, body = windowData.color.header}, anchors = {{from = "TOPRIGHT", object = window.closeBox:GetBody(false), to = "TOPLEFT", x=-4,y=0}}})
		else
			window.collapseBox = nkGFX.Frame:new({id = windowData.id .. 'collapse', parent = window.header:GetBody(), width = 20, height = 20, color = { border = windowData.color.border, body = windowData.color.header}, anchors = {{from = "TOPRIGHT", object = window.header:GetBody(), to = "TOPRIGHT", x=-4,y=4}}})		
		end
		window.collapseBox.Sign = nkGFX.Text:new({id = data.id .. 'collapseSign', text = collapseChar, fontsize = 14, parent = window.collapseBox:GetBody(), color = data.color.border, anchors = {{ from = "CENTER", object = window.collapseBox:GetBody(), to = "CENTER"}}, callBack = function () end})		
		window.collapseBox:SetEvent("LeftClick", function ()
			if data.collapsed == true then 
				if window.footer ~= nil then window.footer:SetVisible(true) end
				window.body:SetVisible(true)
				data.collapsed = false
				window.collapseBox.Sign:SetText('\94')
			else
				if window.footer ~= nil then window.footer:SetVisible(false) end
				window.body:SetVisible(false)				
				data.collapsed = true
				window.collapseBox.Sign:SetText('v')
			end
		end)		
	end	
	
	if data.dragable == true then
		window.header:SetEvent('LeftDown', function(frame)				
				frame.MouseDown = true
				mouseData = Inspect.Mouse()
				if windowData.anchors[1].x == nil then windowData.anchors[1].x = 0 end
				if windowData.anchors[1].y == nil then windowData.anchors[1].y = 0 end
				
				frame.startX = mouseData.x - windowData.anchors[1].x
				frame.startY = mouseData.y - windowData.anchors[1].y
			end)
			
		window.header:SetEvent('MouseMove', function(frame)
				if frame.MouseDown then
					mouseData = Inspect.Mouse()
					curdivX = mouseData.x - frame.startX
					curdivY = mouseData.y - frame.startY
					windowData.anchors[1].x = curdivX
					windowData.anchors[1].y = curdivY
					window:GetBody():SetPoint(windowData.anchors[1].from, windowData.anchors[1].object, windowData.anchors[1].to, curdivX, curdivY)
				end
			end)
		
		window.header:SetEvent('LeftUp', function(frame) 
			if frame.MouseDown then frame.MouseDown = false end 
			if data.dragfunc ~= nil then data.dragfunc (frame:GetLeft(), frame:GetTop()) end
		end)

	end
	
	data.frame = window
	
	setmetatable(data, self)
	self.__index = self
	return data
	
end

function nkGFX.Window:GetBody(flag)

	if flag == false then
		return self.frame:GetBody(false)
	else
		return self.frame.body:GetBody()
	end

end

function nkGFX.Window:SetVisible(flag)

	self.frame:SetVisible(flag)

end

function nkGFX.Window:Collapse(flag)

	local visibility = true
	if flag == true then visibility = false end

	if self.frame.footer ~= nil then self.frame.footer:SetVisible(visibility) end
	self.frame.body:SetVisible(visibility)
	self.collapsed = flag
	
	if flag == false then	
		self.frame.collapseBox.Sign:SetText('\94')		
	else
		self.frame.collapseBox.Sign:SetText('v')
	end
	
end

function nkGFX.CheckBox:new (cbdata)

	if cbdata == nil then return end
	local data = cbdata
	
	if data.template ~= nil then
		for k, value in pairs (data.template) do
			if data[k] == nil then data[k] = value end
		end
	end
	
	if data.checked == nil then data.checked = false end
	
	data.frame = nkGFX.Frame:new({id = data.id, parent = data.parent, anchors = data.anchors })	

	if data.labelinfront == true then
		data.frame.text = nkGFX.Text:new({id = data.id .. 'label', parent = data.frame:GetBody(), wordwrap = false, width = data.labelwidth, text = data.text, fontsize = data.fontsize, color = data.color.border, anchors = {{from = "TOPLEFT", object = data.frame:GetBody(), to = "TOPLEFT" }}  })
		data.frame.box = nkGFX.Frame:new({id = data.id .. 'box', parent = data.frame.text.frame, width = data.fontsize + 2, height = data.fontsize + 2, color = { border = data.color.border, body = data.color.body}, anchors = {{from = "CENTERLEFT", object = data.frame.text.frame, to = "CENTERRIGHT", x =5, y = 0  }}})
	else
		data.frame.box = nkGFX.Frame:new({id = data.id .. 'box', parent = data.frame:GetBody(), wordwrap = false, width = data.fontsize + 2, height = data.fontsize + 2, color = { border = data.color.border, body = data.color.body}, anchors = {{from = "TOPLEFT", object = data.frame:GetBody(), to = "TOPLEFT" }}  })
		data.frame.text = nkGFX.Text:new({id = data.id .. 'label', parent = data.frame.box:GetBody(), width = data.labelwidth, text = data.text, fontsize = data.fontsize, color = data.color.border, anchors = {{from = "CENTERLEFT", object = data.frame.box:GetBody(), to = "CENTERRIGHT", x = 5, y = 0 }}  })
	end
	
	data.frame.box.mark = nkGFX.Frame:new({id = data.id .. 'mark', parent = data.frame.box:GetBody(), width = data.frame.box:GetBody():GetWidth()-4, height = data.frame.box:GetBody():GetHeight()-4, color = { body = data.color.border}, anchors = {{from = "CENTER", object = data.frame.box:GetBody(), to = "CENTER" }}, visible = data.checked})
	data.frame:GetBody():SetWidth(data.frame.text.frame:GetWidth()+ data.frame.box:GetBody():GetWidth()+5)
	data.frame:GetBody():SetHeight(data.frame.box:GetBody():GetHeight())
	
	data.frame.box:SetEvent ("LeftClick", function () 
		if data.checked == false then data.checked = true else data.checked = false end 			
		data:Set(data.checked)
		if data.func ~= nil then data.func (data) end
	end)
	
	setmetatable(data, self)
	self.__index = self
	return data

end

function nkGFX.CheckBox:Set(flag)

	self.checked = flag
	self.frame.box.mark:SetVisible(flag)

end

function nkGFX.CheckBox:GetBody ()

	return self.frame:GetBody()

end

function nkGFX.CheckBox:GetChecked ()

	return self.checked

end

function nkGFX.CheckBox:SetVisible(flag) 
	self.frame:SetVisible(flag) 
end

function nkGFX.Slider:new(sliderData)

	if sliderData == nil then return nil end
	
	local data = sliderData
	
	if data.template ~= nil then
		for k, value in pairs (data.template) do
			data[k] = value
		end
	end
	
	data.frame = nkGFX.Frame:new ({id = data.id, width = data.width, height = 14, parent = data.parent, anchors = data.anchors})

	if data.labelinfront == true then
		data.frame.text = nkGFX.Text:new({id = data.id .. 'label', parent = data.frame:GetBody(), width = data.labelwidth, text = data.text ,fontsize = data.fontsize, color = data.color.border, anchors = {{from = "TOPLEFT", object = data.frame:GetBody(), to = "TOPLEFT" }}  })
		data.frame.bar = nkGFX.Frame:new ({id = data.id .. 'bar', width = data.width, height = 10, parent = data.frame.text.frame, color = data.color, anchors = {{ from = "CENTERLEFT", object = data.frame.text.frame, to = "CENTERRIGHT",x=5,y=0 }}})
	else
		data.frame.bar = nkGFX.Frame:new ({id = data.id .. 'bar', width = data.width, height = 10, parent = data.frame:GetBody(), color = data.color, anchors = {{ from = "TOPLEFT", object = data.frame:GetBody(), to = "TOPLEFT" }}})
		data.frame.text = nkGFX.Text:new({id = data.id .. 'label', parent = data.frame.bar:GetBody(), width = data.labelwidth, text = data.text ,fontsize = data.fontsize, color = data.color.border, anchors = {{from = "CENTERLEFT", object = data.frame.bar:GetBody(), to = "CENTERRIGHT" }}  })		
	end	

	local x = data.width / (data.range[2] - data.range[1]) * (data.value - data.range[1]) - 7
	data.frame.box = nkGFX.Frame:new ({id = data.id .. 'box', width = 14, height = 14, parent = data.frame.bar:GetBody(), color = {body = data.color.border}, anchors = {{from = "CENTERLEFT", object = data.frame.bar:GetBody(), to = "CENTERLEFT", x = x, y = 0}}})

	data.frame.bar:SetEvent('LeftDown', function() data.frame.bar.MouseDown = true end)
		
	data.frame.bar:SetEvent('MouseMove', function()
		if data.frame.bar.MouseDown then
			if Inspect.Mouse().y < (data.frame.bar:GetBody():GetTop()-10) or Inspect.Mouse().y > (data.frame.bar:GetBody():GetTop() + 20) then data.frame.bar.MouseDown = false end
			if Inspect.Mouse().x < (data.frame.bar:GetBody():GetLeft()-20) or Inspect.Mouse().x > (data.frame.bar:GetBody():GetLeft() + data.width + 20) then data.frame.bar.MouseDown = false end
			local curdivX = Inspect.Mouse().x - data.frame.bar:GetBody():GetLeft()-7
			local valuePerPixel = (data.range[2] - data.range[1]) / data.width
			local newValue = curdivX * valuePerPixel + data.range[1]
			if newValue < data.range[1] then newValue = data.range[1] end
			if newValue > data.range[2] then  newValue = data.range[2] end
			data:Set (newValue)
			if data.func ~= nil then data.func (data) end
		end		
	end)
	
	data.frame.bar:SetEvent('LeftUp', function() data.frame.bar.MouseDown = false end)
	
	setmetatable(data, self)
	self.__index = self
	return data

end

function nkGFX.Slider:GetBody ()

	return self.frame:GetBody()

end

function nkGFX.Slider:Set (value)

	self.value = value
	local x = self.width / (self.range[2] - self.range[1]) * ( value - self.range[1]) - 7
	self.frame.box:GetBody():SetPoint ("CENTERLEFT", self.frame.bar:GetBody(),"CENTERLEFT", x, 0)

end

function nkGFX.RadioButton:new (rbData)

	if rbData == nil then return nil end
	local data = rbData
	
	if data.template ~= nil then
		for k, value in pairs (data.template) do
			if data[k] == nil then data[k] = value end
		end
	end
	
	data.frame = nkGFX.Frame:new({id = data.id, parent = data.parent, height = 16, anchors = data.anchors })	
	data.frame.label = nkGFX.Text:new({id = data.id .. 'label', parent = data.frame:GetBody(), anchors = {{from = "TOPLEFT", object = data.frame:GetBody(), to = "TOPLEFT"}}, color = data.color.border, text = data.label, fontsize = data.fontsize, width = data.labelwidth})	
	
	data.frame.cbOptions = {}
	
	local cbData = data
	
	local callBackFunc = function (thisCheckbox)
		for k, checkbox in pairs (data.frame.cbOptions) do
			if checkbox.id ~= thisCheckbox.id then 
				checkbox.checked = false
				checkbox.frame.box.mark:SetVisible(false) 
			end
		end
		
		if thisCheckbox:GetChecked() == false then thisCheckbox:Set(true) end
		if data.func ~= nil then
			local value = nil
			for k, details in pairs (data.options) do
				if details.label == thisCheckbox.frame.text.frame:GetText() then
					value = details.value
					break
				end
			end
			data.func(value)
		end
	end
	
	for k, details in pairs (data.options) do
		
		local anchors = {{from = "CENTERLEFT", object = data.frame.label.frame, to = "CENTERRIGHT"}}
		if k~=1 then anchors[1].object = data.frame.cbOptions[k-1]:GetBody(false) end

		local checked = false
		if k == 1 and data.value == nil then 
			checked = true
		elseif details.value == data.value then
			checked = true
		end
		
		local cbOption = nkGFX.CheckBox:new ({id = data.id .. 'cb' .. k, parent = data.frame:GetBody(), anchors = anchors, text = details.label, labelinfront=false, labelwidth = data.optionwidth, template = data.template, func = callBackFunc, checked = checked})
		table.insert(data.frame.cbOptions, cbOption)

	end
	
	setmetatable(data, self)
	self.__index = self
	return data
	
end

function nkGFX.RadioButton:Set (value)

	if value == nil then return end
	local curValue = self:GetValue()
	local validValue = false
	
	for k, details in pairs (self.options) do
		if details.value == value then
			validValue = true
			self.frame.cbOptions[k]:Set(true)
		else
			self.frame.cbOptions[k]:Set(false)
		end
	end
	
	if validValue == false then nkGFX.RadioButton:Set (curValue) end

end

function nkGFX.RadioButton:GetValue()

	for k, checkbox in pairs (self.frame.cbOptions) do
		if checkbox:GetChecked() == true then return self.options[k] end
	end
	
	return nil

end

function nkGFX.RadioButton:GetBody()

	return self.frame:GetBody()

end

function nkGFX.Texture:new (settings)

	if settings == nil then return end
	local data = settings
	
	data.texture = UI.CreateFrame("Texture", data.id, data.parent)
	
	if data.width ~= nil then data.texture:SetWidth(data.width)  end
	if data.height ~= nil then data.texture:SetHeight(data.height) end
	
	if data.anchors ~= nil then
		for k, anchor in pairs(data.anchors) do
			if anchor.x ~= nil and anchor.y ~= nil then
				data.texture:SetPoint (anchor.from, anchor.object, anchor.to, anchor.x, anchor.y)
			else
				data.texture:SetPoint (anchor.from, anchor.object, anchor.to)
			end
		end
	end
	
	if data.callBack ~= nil then
		function data.texture.Event:LeftClick()
			data.callBack(self)
		end
	end
	
	if data.visible == false then data:SetVisible(false) end
		
	setmetatable(data, self)
	self.__index = self
	return data

end

function nkGFX.Texture:Set(settings) 

	if settings == nil then return end
	self.texture:SetTexture(settings.type, settings.texture)

end

function nkGFX.Texture:Update (settings)

	if settings == nil then return end
	for k, v in pairs (settings) do self[k] = v end
	
	if self.width ~= nil then self.texture:SetWidth(self.width) end
	if self.height ~= nil then self.texture:SetHeight(self.height) end

end

function nkGFX.Textfield:new(settings)

	if settings == nil then return end
	local data = settings
	
	data.frame = nkGFX.Frame:new({ id = data.id .. "border", width = data.width, height = 20, color = data.color, anchors = data.anchors, parent = data.parent })	
	
	data.frame.input = UI.CreateFrame("RiftTextfield", data.id, data.frame:GetBody())
	data.frame.input:SetPoint ("CENTERLEFT", data.frame:GetBody(), "CENTERLEFT", 1, 1)	
	if data.value ~= nil then data.frame.input:SetText (data.value) end
	if data.width ~= nil then data.frame.input:SetWidth(data.width) end
	
	setmetatable(data, self)
	self.__index = self
	return data
	
end

function nkGFX.Textfield:GetValue()

	return self.frame.input:GetText()

end